/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const traverse=require("ast-traverse");const logging=require("../utilities/logging");const generate=require("@babel/generator").default;const{parse}=require("@babel/parser");const self=require("./astUtils");function inManifestData(tag){for(let obj of global.manifestData){if(obj.componentName===tag){return obj}}return false}function checkFileUpdate(){if(!global.fileIsUpdated){logging.logFilePath(global.filePath);global.fileIsUpdated=true;global.migrationMetrics.files+=1;return true}return false}function generateAST(code){try{return parse(code,{sourceType:"unambiguous",plugins:global.plugins})}catch(e){if(e instanceof SyntaxError){logging.logFilePath(global.filePath,true);logging.logMessage(`\tCouldn't migrate file because file syntax is not clear`);global.errorInFile=true}}}function generateCode(ast){return generate(ast,{retainLines:false}).code}function _generateSpecCode(ast){return generateCode(ast).replace(/^\s*$(?:\r\n?|\n)/gm,"")}function _appendAttribute(name,value,tag){for(let attr of tag.attributes){if(attr.name.name===name){return false}}let node;if(!value.expression){node={type:"StringLiteral",value:`${value}`}}else{const expAst=generateAST(`{${value.expression}}`).program.body[0].body[0].expression;node={type:"JSXExpressionContainer",expression:expAst}}const attrNode={type:"JSXAttribute",name:{type:"JSXIdentifier",name:`${name}`},value:node};tag.attributes.push(attrNode);return true}function appendImport(value,index,ast){const importNode={type:"JSXText",value:`import "${value}";\n`};ast.program.body.splice(index,0,importNode)}function _appendImportToRequire(toImport,node,parent){const importNode={type:"StringLiteral",value:`${toImport}`};if(parent){parent.elements.push(importNode)}else{node.expression.arguments.push(importNode)}}function _hasImportInRequire(args,fromImport){for(let arg of args){if(arg.type==="ArrayExpression"){obj=_hasImportInRequire(arg.elements,fromImport);if(obj&&!obj.parent){obj.parent=arg}return obj}else if(arg.type==="StringLiteral"&&arg.value===fromImport){return{child:arg}}}return false}function _hasImport(body,importVal,newImports){for(let node of body){if(node.type==="ImportDeclaration"&&node.source.value===importVal||node.type==="JSXText"&&node.value.includes(importVal)){return true}}for(let obj of newImports){if(obj.name===importVal){return true}}return false}function _commentJSX(node){return node.type==="JSXElement"?`{ /* ${_generateSpecCode(node)} */ }`:`{ /* ${node.value} */ }`}function _getText(text,locObj){let specificText;const dataArray=text.split("\n");const{start,end}=locObj;if(start.line===end.line){specificText=dataArray[start.line-1].substring(start.column,end.column);return specificText}else{dataArray[start.line-1]=dataArray[start.line-1].substring(start.column);const textByLines=dataArray.slice(start.line-1,end.line).map(s=>s.trim());specificText=textByLines.join("\n");return specificText}}function _getLabelFromLegacyComponent(node,slotName,labelVal="",cnodesArr=[]){for(let child of node.children){switch(child.type){case"JSXText":if(/\S/.test(child.value)){if(labelVal){if(labelVal.expression){labelVal.expression=`(${labelVal.expression}) + ' ${child.value.trim()}'`}else{labelVal+=" "+child.value.trim()}}else{labelVal=child.value.trim()}child.type="JSXText";child.value="";cnodesArr.push(child)}break;case"JSXExpressionContainer":if(child.expression&&child.expression.type!="JSXEmptyExpression"){if(labelVal){if(labelVal.expression){labelVal.expression=`(${labelVal.expression}) + (${_generateSpecCode(child.expression)})`}else{labelVal={expression:`'${labelVal} ' + (${_generateSpecCode(child.expression)})`}}}else{labelVal={expression:_generateSpecCode(child.expression)}}child.type="JSXText";child.value="";cnodesArr.push(child)}break;case"JSXElement":let slotValue;switch(child.openingElement.name.name.toUpperCase()){case"OJ-BIND-TEXT":for(let attr of child.openingElement.attributes){if(attr.name.name=="value"){if(attr.value.type==="JSXExpressionContainer"){if(labelVal.expression){labelVal.expression=`${labelVal.expression} + (${_generateSpecCode(attr.value.expression)})`}else{if(labelVal){labelVal={expression:`'${labelVal} ' + (${_generateSpecCode(attr.value.expression)})`}}else{labelVal={expression:_generateSpecCode(attr.value.expression)}}}}else if(attr.value.type==="StringLiteral"){if(labelVal.expression){labelVal.expression=`${labelVal.expression} + ' ${attr.value.value}'`}else{if(labelVal){labelVal+=" "+attr.value.value}else{labelVal=attr.value.value}}}child.type="JSXText";child.value="";cnodesArr.push(child);break}}break;case"SPAN":case"DIV":child.openingElement.attributes.forEach(attr=>{if(attr.name.name=="slot"&&(attr.value.value===slotName||!attr.value.value.length))slotValue=true});if(!child.openingElement.attributes.map(att=>att.name.name).includes("slot")){slotValue=true}if(slotValue){const result=_getLabelFromLegacyComponent(child,slotName,labelVal,cnodesArr);let emptyChildren=true;let i=0;var regex=new RegExp("^[^\\dA-Za-z]*$");while(emptyChildren===true&&child.children[i]){if(!(child.children[i].type==="JSXText"&&regex.test(child.children[i].value))){emptyChildren=false}i++}if(emptyChildren){result.cnodes.push(child)}return{label:result.label,cnodes:result.cnodes}}break}}}return{label:labelVal,cnodes:cnodesArr}}function _findPeer(ast,node,peerTag,componentAttr,peerAttr){const attributeValue=_findAttribute(node.openingElement.attributes,componentAttr).attrValue;let peer;if(attributeValue){traverse(ast,{pre:function(n){if(n.type==="JSXElement"&&n.openingElement.name&&n.openingElement.name.name===peerTag){if(_findAttribute(n.openingElement.attributes,peerAttr,attributeValue).attrValue===attributeValue){peer=n}}}});return peer}}function _findVariable(ast,name){let variable;traverse(ast,{pre:function(n){if(n.type==="VariableDeclarator"&&n.id.name===name){variable=n}}});return variable}function _hasAttributesToIgnoreHelper(node,rule,key,parent,ast,corePackTag){let keys;if(rule.missPeerTag){const peerRule=rule.missPeerTag;for(let tag of peerRule.tag){for(let attrSet of peerRule.matchValues){const peer=_findPeer(ast,node,tag,attrSet.ownAttribute,attrSet.peerAttribute);if(peer&&(!peerRule.attribute||!_findAttribute(peer.openingElement.attributes,peerRule.attribute))&&(!peerRule.missSlot||_findAttribute(peer.openingElement.attributes,"slot",peerRule.missSlot))){return false}}}return true};if(rule.missDescendant){const descRule=rule.missDescendant;for(let tag of descRule.tag){const desc=_findDescendant(node,tag,descRule.missSlot);if(desc&&(!descRule.attribute||!_findAttribute(desc.openingElement.attributes,descRule.attribute))){return false}}return true};if(rule.missAttribute){keys=Object.keys(rule.missAttribute);for(let key in rule.missAttribute){rule.missAttribute[_toCamelCase(key)]=rule.missAttribute[key];keys.push(_toCamelCase(key))}}if(!rule.descendant&&node.openingElement.attributes.length){for(let i=0;i<node.openingElement.attributes.length;i++){const attribute=node.openingElement.attributes[i];if(rule.missAttribute){if(keys.includes(attribute.name.name)&&(!rule.missAttribute[attribute.name.name]||attribute.value.value===rule.missAttribute[attribute.name.name])){return false}}else{if(rule.attribute){const hasAttribute=_findAttribute(attribute,rule.attribute);if(hasAttribute&&(rule.allowedValues&&!rule.allowedValues.includes(attribute.value.value)||rule.prohibitedValues&&rule.prohibitedValues.includes(attribute.value.value)||!rule.allowedValues&&!rule.prohibitedValues)){const value=rule.allowedValues||rule.prohibitedValues?attribute.value.value:"";if(corePackTag){if(rule.typeDefined){const variable=_findVariable(ast,hasAttribute.attrValue.expression?hasAttribute.attrValue.expression.expression.name:hasAttribute.attrValue);if(variable&&variable.id.typeAnnotation){checkFileUpdate();logging.logIgnoreComponent(node.openingElement.name.name,hasAttribute.format?hasAttribute.attribute:rule.attribute,"",parent,"",`Component needs manual migration due to a legacy type`);return true}return false}else{checkFileUpdate();logging.logIgnoreComponent(node.openingElement.name.name,hasAttribute.format?hasAttribute.attribute:rule.attribute,value,parent)}}return true}}if(attribute.name.name===key&&attribute.value.value===rule[key]){if(corePackTag){checkFileUpdate();logging.logIgnoreComponent(node.openingElement.name.name,key,rule[key],parent)}return true}}}}if(!rule.descendant&&rule.missAttribute){if(corePackTag){checkFileUpdate();logging.logIgnoreComponent(node.openingElement.name.name,rule.missAttribute,"",parent,true)}return true}if(node.children){for(let j=0;j<node.children.length;j++){let child=node.children[j];if(child.type==="JSXElement"){if(rule.descendant){if(child.openingElement.name&&child.openingElement.name.name===rule.descendant){if(rule.attribute||rule.slotName||rule.missSlot){for(let attr of child.openingElement.attributes){if(rule.attribute){const hasAttribute=_findAttribute(attr,rule.attribute);if(hasAttribute&&(rule.allowedValues&&!rule.allowedValues.includes(attr.value.value)||rule.prohibitedValues&&rule.prohibitedValues.includes(attr.value.value)||!rule.prohibitedValues&&!rule.allowedValues)){if(hasAttribute.format){rule.attribute=hasAttribute.attribute}if(corePackTag){checkFileUpdate();logging.logIgnoreComponent(child.openingElement.name.name,rule.attribute,attr.value.value,parent)}return true}}else if(attr.name.name==="slot"){if(rule.slotName&&rule.slotName===attr.value.value){if(corePackTag){checkFileUpdate();logging.logIgnoreComponent(child.openingElement.name.name,"slot",attr.value.value,parent)}return true}else if(rule.missSlot&&attr.value.value===rule.missSlot){return false}}}if(rule.missSlot){if(corePackTag){checkFileUpdate();logging.logIgnoreComponent(child.openingElement.name.name,"slot",rule.missSlot,parent,true)}return true}}else{if(corePackTag){checkFileUpdate();logging.logIgnoreComponent(rule.descendant,"","",parent)}return true}}else if(_hasAttributesToIgnoreHelper(child,rule,key,parent,ast,corePackTag)){return true}}if(child.openingElement.attributes&&!rule.attribute&&!rule.missAttribute&&_hasAttributesToIgnoreHelper(child,rule,key,parent,ast,corePackTag)){if(corePackTag){checkFileUpdate();logging.logIgnoreComponent(node.openingElement.name.name,key,rule[key])}return true}}}}}function hasAttributesToIgnore(node,metadata,ast,parent=""){const ignoreRuleData=metadata.ruleSetToIgnore;if(node.openingElement){if(ignoreRuleData){for(let rule of ignoreRuleData){let arr=Object.keys(rule);let key=arr[0];if(rule.descendant){parent=node.openingElement.name.name}if(_hasAttributesToIgnoreHelper(node,rule,key,parent,ast,metadata.corePackTag)){return true}}}}return false}function hasExpressionAttributes(component,componentName,code,importantAttributes=null,isPre=false){const excludedAttributes=global.configData.excludeAttributeExpressions[componentName];if(global.jaf.options.dryRunMode&&importantAttributes){component.attributes.forEach(attribute=>{if(importantAttributes.includes(attribute.name.name)&&attribute.value&&attribute.value.type==="JSXExpressionContainer"){const expression=_getText(code,attribute.value.loc);if(checkFileUpdate()){logging.logAttributeExpression(componentName,attribute.name.name,expression)}}})}if(excludedAttributes&&excludedAttributes.length){for(let exAtt of excludedAttributes){for(let i=0;i<component.attributes.length;i++){const attribute=component.attributes[i];if(exAtt===attribute.name.name&&attribute.value.type==="JSXExpressionContainer"){const expression=attribute.value.loc?_getText(code,attribute.value.loc):_generateSpecCode(attribute.value);checkFileUpdate();logging.logExcludeAttributeExpression(componentName,exAtt,expression);return true}}}}return false}function commentSlotContent(node,data){for(let child of node.children){if(child.openingElement&&child.openingElement.attributes){for(let attribute of child.openingElement.attributes){if(attribute.name.name=="slot"&&attribute.value.value==data.slot){const value=_commentJSX(child);index=node.children.indexOf(child);node.children[index]=generateAST(_generateSpecCode({type:"JSXText",value:`${value}`}));checkFileUpdate();return[logging.logCommentSlotContent(node.openingElement.name.name,data.slot)]}};}};}function _transferAttribute(fromTag,toTag,attribute,value){for(let attr in fromTag.attributes){if(fromTag.attributes[attr].name.name===attribute&&(fromTag.attributes[attr].value.value===value||value.expression&&_generateSpecCode(fromTag.attributes[attr].value.expression)===value.expression)){_appendAttribute(attribute,value,toTag);fromTag.attributes.splice(attr,1);break}}}function _findDescendant(node,tagName,slotName){if(node.children){for(let child of node.children){if(child.type==="JSXElement"&&child.openingElement.name&&child.openingElement.name.name===tagName){if(!slotName){return child}else{for(let attr of child.openingElement.attributes){if(attr.name.name==="slot"&&attr.value.value===slotName){return child}}}}else{const descendant=_findDescendant(child,tagName,slotName?slotName:"");if(descendant){return descendant}}}}return false}function slotContentToAttribute(node,data,code,ast){let logs=[];for(let obj of data){let toNode;if(obj.toDescendant){const descendant=_findDescendant(node,obj.toDescendant,obj.slotNameTo?obj.slotNameTo:"");if(descendant){toNode=descendant}else{continue}}if(obj.fromDescendant){const descendant=_findDescendant(node,obj.fromDescendant,obj.slotNameFrom?obj.slotNameFrom:"");if(descendant){node=descendant}else{continue}}if(obj.toPeer||obj.fromPeer){const peer=_findPeer(ast,node,obj.toPeer?obj.toPeer:obj.fromPeer,obj.ownAttribute,obj.peerAttribute);if(peer){if(obj.toPeer){toNode=peer}else{node=peer}}else{continue}}if(!node.openingElement.attributes.map(att=>att.name.name).includes(obj.attribute)){const component=node.openingElement;const legacyButtonLabel=_getLabelFromLegacyComponent(node,obj.slotName);if(legacyButtonLabel){let{label,cnodes}=legacyButtonLabel;if(label){for(let cnode of cnodes){if(cnode.type==="JSXElement"&&(cnode.openingElement.name.name==="span"||cnode.openingElement.name.name==="div")){for(let child of cnode.children){if(!child.type==="JSXText"){cnode.value="";cnode.type="JSXText"}}}cnode.value="";cnode.type="JSXText"}checkFileUpdate();_appendAttribute(obj.attribute,label,component);logs.push(logging.logSlotContentToAttribute(node.openingElement.name.name,obj.attribute,label.expression?`{${label.expression}}`:label));if(toNode){_transferAttribute(component,toNode.openingElement,obj.attribute,label);logs.push(logging.logAttributeTransfer(component.name.name,toNode.openingElement.name.name,obj.attribute,label.expression?`{${label.expression}}`:label))}return logs}}}}}function classNameToAttribute(node,data){let logs=[];for(let obj of data){for(let attr of node.openingElement.attributes){if(attr.name.name=="class"&&attr.value?.value?.includes(obj.className)){if(obj.attribute){classNames=attr.value.value.split(" ");index=classNames.indexOf(obj.className);classNames.splice(index,1);if(classNames.length===0){attr.name.name=`${obj.attribute}`;attr.value.value=`${obj.attributeValue}`}else{attr.value.value=`${classNames.join(" ")}`;_appendAttribute(obj.attribute,obj.attributeValue,node.openingElement)}checkFileUpdate();logs.push(logging.logClassNameToAttribute(obj.className,obj.attribute,obj.attributeValue))}break}}}return logs}function handleSpecialCase(code,data){let obj;let regex;if(data[1]){obj=data[1];regex=new RegExp(obj.fromString.split(" ").join("\\s*"),"igm");code=code.replace(regex,obj.toString)}obj=data[0];regex=new RegExp(obj.fromString.split(" ").join("\\s*"),"igm");code=code.replace(regex,obj.toString);return code}function migrateTag(node,legacyComponentName,coreComponentName,isChild){node.openingElement.name.name=coreComponentName;checkFileUpdate();global.migrationMetrics.components+=1;if(node.closingElement&&node.closingElement.name.name===legacyComponentName){node.closingElement.name.name=coreComponentName}return logging.logTagMigration(legacyComponentName,coreComponentName,isChild)}function migrateTagByAttribute(node,data,isChild){for(const tagSet of data){if(tagSet.attribute){let attribute=_findAttribute(node.openingElement.attributes,tagSet.attribute,tagSet.attributeValue);if(attribute){node.openingElement.name.name=tagSet.toTag;checkFileUpdate();global.migrationMetrics.components+=1;if(node.closingElement&&node.closingElement.name.name===tagSet.fromTag){node.closingElement.name.name=tagSet.toTag}return logging.logTagMigration(tagSet.fromTag,tagSet.toTag)}}else{node.openingElement.name.name=tagSet.toTag;checkFileUpdate();global.migrationMetrics.components+=1;if(node.closingElement&&node.closingElement.name.name===tagSet.fromTag){node.closingElement.name.name=tagSet.toTag}return logging.logTagMigration(tagSet.fromTag,tagSet.toTag,isChild)}}}function migrateImports(ast,metadata,code,newImports=[]){let importIndex;let importTag="";let importUpdated=false;const importData=metadata.migrateImports;const corePackTag=metadata.corePackTag;const legacyTag=metadata.legacyTag;const ruleSetToIgnore=metadata.ruleSetToIgnore;if(global.plugins[0]!="jsx"||Array.isArray(corePackTag)&&corePackTag.some(s=>code.includes(`<${s}`))||code.includes(`<${corePackTag}`)){if(ruleSetToIgnore&&ruleSetToIgnore.length>0&&!global.configData.forceMigration.includes(legacyTag)&&(global.plugins[0]!="jsx"||code.includes(`<${legacyTag}`))){if(Array.isArray(importData.to)&&importData.to.length>0&&!importData.to.some(s=>_hasImport(ast.program.body,s,newImports))){traverse(ast,{pre:function(node){if(node.type=="ImportDeclaration"&&node.source.value===importData.from&&!node.specifiers.length){for(let i=0;i<importData.to.length;i++){importTag=`<${importData.to[i].replace("/","-")}`;if(!_hasImport(ast.program.body,importData.to[i],newImports)&&(global.plugins[0]!="jsx"||code.includes(importTag))){importIndex=ast.program.body.indexOf(node);newImports.push({name:importData.to[i],index:importIndex});checkFileUpdate();logging.logImportAppend(importData.to[i],importData.from)}}}else if(node.type=="ExpressionStatement"&&node.expression.callee&&(node.expression.callee.name=="require"||node.expression.callee.name=="define")){const fromImportLoc=_hasImportInRequire(node.expression.arguments,importData.from);if(fromImportLoc){const parent=fromImportLoc.parent?fromImportLoc.parent:false;for(let i=0;i<importData.to.length;i++){importTag=`<${importData.to[i].replace("/","-")}`;if(!_hasImportInRequire(node.expression.arguments,importData.to[i])&&(global.plugins[0]!="jsx"||code.includes(importTag))){_appendImportToRequire(importData.to[i],node,parent);checkFileUpdate();logging.logImportAppend(importData.to[i],importData.from);newImports=false}}}}}})}else if(!Array.isArray(importData.to)&&!_hasImport(ast.program.body,importData.to,newImports)){traverse(ast,{pre:function(node){if(node.type=="ImportDeclaration"&&node.source.value===importData.from&&!node.specifiers.length&&!_hasImport(ast.program.body,importData.to,newImports)){importIndex=ast.program.body.indexOf(node);newImports.push({name:importData.to,index:importIndex});checkFileUpdate();logging.logImportAppend(importData.to,importData.from)}else if(node.type=="ExpressionStatement"&&node.expression.callee&&(node.expression.callee.name=="require"||node.expression.callee.name=="define")){const fromImportLoc=_hasImportInRequire(node.expression.arguments,importData.from);if(fromImportLoc&&!_hasImportInRequire(node.expression.arguments,importData.to)){const parent=fromImportLoc.parent?fromImportLoc.parent:false;_appendImportToRequire(importData.to,node,parent);checkFileUpdate();logging.logImportAppend(importData.to,importData.from);newImports=false}}}})}}else{if(Array.isArray(importData.to)&&importData.to.length>0&&!importData.to.some(s=>_hasImport(ast.program.body,s,newImports))){traverse(ast,{pre:function(node){if(node.type=="ImportDeclaration"&&node.source.value===importData.from&&!node.specifiers.length){importTag=`<${importData.to[0].replace("/","-")}`;if(!_hasImport(ast.program.body,importData.to[0],newImports)&&(global.plugins[0]!="jsx"||code.includes(importTag))){importUpdated=true;node.source.value=importData.to[0];checkFileUpdate();logging.logImportUpdate(importData.from,importData.to[0])}for(let i=1;i<importData.to.length;i++){importTag=`<${importData.to[i].replace("/","-")}`;if(!_hasImport(ast.program.body,importData.to[i],newImports)&&(global.plugins[0]!="jsx"||code.includes(importTag))){if(importUpdated){importIndex=ast.program.body.indexOf(node);newImports.push({name:importData.to[i],index:importIndex});logging.logImportAppend(importData.to[i],importData.from)}else{importUpdated=true;node.source.value=importData.to[i];logging.logImportUpdate(importData.from,importData.to[i])}checkFileUpdate()}}}else if(node.type=="ExpressionStatement"&&node.expression.callee&&(node.expression.callee.name=="require"||node.expression.callee.name=="define")){const fromImportLoc=_hasImportInRequire(node.expression.arguments,importData.from);if(fromImportLoc){const parent=fromImportLoc.parent?fromImportLoc.parent:false;importTag=`<${importData.to[0].replace("/","-")}`;if(!_hasImportInRequire(node.expression.arguments,importData.to[0])&&(global.plugins[0]!="jsx"||code.includes(importTag))){importUpdated=true;fromImportLoc.child.value=importData.to[0];checkFileUpdate();logging.logImportUpdate(importData.from,importData.to[0]);newImports=false}for(let i=1;i<importData.to.length;i++){importTag=`<${importData.to[i].replace("/","-")}`;if(!_hasImportInRequire(node.expression.arguments,importData.to[i])&&(global.plugins[0]!="jsx"||code.includes(importTag))){if(importUpdated){_appendImportToRequire(importData.to[i],node,parent);logging.logImportAppend(importData.to[i],importData.from)}else{importUpdated=true;fromImportLoc.child.value=importData.to[i];logging.logImportUpdate(importData.from,importData.to[i])}checkFileUpdate()}}}}}})}else if(!Array.isArray(importData.to)&&!_hasImport(ast.program.body,importData.to,newImports)){traverse(ast,{pre:function(node){if(node.type=="ImportDeclaration"&&!node.specifiers.length&&node.source.value===importData.from&&!_hasImport(ast.program.body,importData.to,newImports)){node.source.value=importData.to;checkFileUpdate();logging.logImportUpdate(importData.from,importData.to);newImports=false}else if(node.type=="ExpressionStatement"&&node.expression.callee&&(node.expression.callee.name=="require"||node.expression.callee.name=="define")&&!_hasImportInRequire(node.expression.arguments,importData.to)){const fromImportLoc=_hasImportInRequire(node.expression.arguments,importData.from);if(fromImportLoc){fromImportLoc.child.value=importData.to;checkFileUpdate();logging.logImportUpdate(importData.from,importData.to);newImports=false}}}})}}if(Array.isArray(corePackTag)){for(tag of corePackTag){const startTag="<"+tag;if(code.includes(startTag)&&(Array.isArray(importData.to)&&!importData.to.some(s=>_hasImport(ast.program.body,s,newImports))||!Array.isArray(importData.to)&&!_hasImport(ast.program.body,importData.to,newImports))){if(Array.isArray(importData.to)){importData.to.forEach(toImport=>{importTag=`<${toImport.replace("/","-")}`;if(global.plugins[0]!="jsx"||code.includes(importTag)){newImports.push({name:toImport,index:0});checkFileUpdate();logging.logMissingImport(tag,toImport)}})}else{newImports.push({name:importData.to,index:0});checkFileUpdate();logging.logMissingImport(tag,importData.to)}break}}}else{startTag="<"+corePackTag;if(code.includes(startTag)&&(Array.isArray(importData.to)&&!importData.to.some(s=>_hasImport(ast.program.body,s,newImports))||!Array.isArray(importData.to)&&!_hasImport(ast.program.body,importData.to,newImports))){if(Array.isArray(importData.to)){importData.to.forEach(toImport=>{importTag=`<${toImport.replace("/","-")}`;if(global.plugins[0]!="jsx"||code.includes(importTag)){newImports.push({name:toImport,index:0});checkFileUpdate();logging.logMissingImport(toImport.replace("/","-"),toImport)}})}else{newImports.push({name:importData.to,index:0});checkFileUpdate();logging.logMissingImport(importData.to.replace("/","-"),importData.to)}}}}if(global.plugins[0]==="jsx"){startTag="<"+legacyTag;if((code.includes(startTag+" ")||code.includes(startTag+">"))&&!_hasImport(ast.program.body,importData.from,newImports)){newImports.push({name:importData.from,index:0});checkFileUpdate();logging.logMissingImport(legacyTag,importData.from)}}return newImports}function _toCamelCase(attribute){let camelCaseName=attribute.split("-");for(let i in camelCaseName){if(i>0){camelCaseName[i]=camelCaseName[i][0].toUpperCase()+camelCaseName[i].slice(1)}}camelCaseName=camelCaseName.join("");return camelCaseName}function _findAttribute(attribs,name,value=""){if(name){const camelCaseName=_toCamelCase(name);if(Array.isArray(attribs)){for(let attr of attribs){if(attr.name.name===name||attr.name.name===`:${name}`||attr.name.name===camelCaseName){if(value&&attr.value.value!=value){return false}if(attr.name.name===name){let attrValue="";if(attr.value){if(attr.value.type==="JSXExpressionContainer"){attrValue={expression:attr.value}}else{attrValue=attr.value.value}}return{"attribute":name,"attrValue":attrValue}}else if(attr.name.name===camelCaseName){let attrValue="";if(attr.value){if(attr.value.type==="JSXExpressionContainer"){attrValue={expression:attr.value}}else{attrValue=attr.value.value}}return{"attribute":camelCaseName,"format":"camel","attrValue":attrValue}}}}}else{if(attribs.name.name===name||attribs.name.name===camelCaseName){if(value&&!(attribs.value.value===value)){return false}if(attribs.name.name===name){let attrValue="";if(attribs.value){if(attribs.value.type==="JSXExpressionContainer"){attrValue={expression:attribs.value}}else{attrValue=attribs.value.value}}return{"attribute":name,"attrValue":attrValue}}else if(attribs.name.name===camelCaseName){let attrValue="";if(attribs.value){if(attribs.value.type==="JSXExpressionContainer"){attrValue={expression:attribs.value}}else{attrValue=attribs.value.value}}return{"attribute":camelCaseName,"format":"camel","attrValue":attrValue}}}}}return false}function migrateAttributes(node,data,code,ast,legacyTag){let logs=[];for(const attributeSet in data){let toNode;let tagExists=true;let component=node.openingElement;let obj=data[attributeSet];if(obj.fromDescendant||obj.fromPeer||obj.toDescendant||obj.toPeer){tagExists=false;if(obj.fromDescendant||obj.toDescendant){if(node.children){const descendant=_findDescendant(node,obj.fromDescendant?obj.fromDescendant:obj.toDescendant,obj.slotName);if(descendant){tagExists=true;if(obj.fromDescendant){component=descendant.openingElement;toNode=node.openingElement}else{toNode=descendant.openingElement}}else{continue}}}else{const peerTag=_findPeer(ast,node,obj.toPeer?obj.toPeer:obj.fromPeer,obj.ownAttribute,obj.peerAttribute);if(peerTag){tagExists=true;if(obj.fromPeer){component=peerTag.openingElement;toNode=node.openingElement}else{toNode=peerTag.openingElement}}else{continue}}}if(tagExists){let hasDepAttribute=true;if(obj.depAttribute){hasDepAttribute=_findAttribute(component.attributes,obj.depAttribute)}const hasToAttribute=_findAttribute(component.attributes,obj.toAttribute);component.attributes.forEach(attribute=>{const hasFromAttribute=_findAttribute(attribute,obj.fromAttribute,obj.fromValue);if(hasFromAttribute.format){obj.fromAttribute=hasFromAttribute.attribute;obj.toAttribute=_toCamelCase(obj.toAttribute)}if(!(obj.fromAttribute===obj.toAttribute&&obj.fromValue===obj.toValue)&&!obj.fromKeyValue){if(hasDepAttribute&&hasFromAttribute&&obj.toAttribute){if(obj.toValue){attribute.name.name=obj.toAttribute;attribute.value.value=obj.toValue;checkFileUpdate();logs.push(logging.logAttributeMigration(obj.fromAttribute,obj.fromValue,obj.toAttribute,obj.toValue))}else{const attValue=attribute.value.value||{expression:_getText(code,attribute.value.loc)};if(obj.fromValue){attribute.name.name=obj.toAttribute;attribute.value.value="";checkFileUpdate();logs.push(logging.logAttributeMigration(obj.fromAttribute,attValue.expression||attValue,obj.toAttribute,obj.toValue))}else{attribute.name.name=obj.toAttribute;attribute.value.value=attValue.expression||attValue;checkFileUpdate();logs.push(logging.logAttributeMigration(obj.fromAttribute,attValue.expression||attValue,obj.toAttribute))}}}else if(hasDepAttribute&&hasFromAttribute&&!obj.toAttribute){index=component.attributes.indexOf(attribute);component.attributes.splice(index,1);checkFileUpdate();logs.push(logging.logAttributeDeprecation(obj.fromAttribute,obj.fromValue))}}if(hasFromAttribute&&obj.fromKeyValue&&attribute.value.type==="JSXExpressionContainer"){let inline=true;let variable;if(attribute.value.expression.type==="Identifier"){inline=false;const varName=attribute.value.expression.name;variable=_findVariable(ast,varName)}if(variable&&variable.init.elements||attribute.value.expression.elements){for(let elem of inline?attribute.value.expression.elements:variable.init.elements){if(elem.properties){for(let prop of elem.properties){const fromKeys=Object.keys(obj.fromKeyValue);const toKeys=Object.keys(obj.toKeyValue);for(let key in fromKeys){if((prop.key.name?prop.key.name:prop.key.value)===fromKeys[key]){if(!obj.fromKeyValue[fromKeys[key]]||obj.fromKeyValue[fromKeys[key]]===prop.value.value){if(prop.key.name){prop.key.name=toKeys[key]}else if(prop.key.value){prop.key.value=toKeys[key]}const fromVal=prop.value.value;if(obj.toKeyValue[toKeys[key]]){prop.value.type="StringLiteral";prop.value.value=obj.toKeyValue[toKeys[key]]}checkFileUpdate();logs.push(logging.logObjectManipulation(attribute.name.name,fromKeys[key],prop.key.name?prop.key.name:prop.key.value,fromVal,prop.value.value))}}}}}else{const log=logging.logAttributeWarning(obj.fromAttribute);if(!logs.includes(log)){logs.push(logging.logAttributeWarning(obj.fromAttribute));checkFileUpdate()}break}}}else{checkFileUpdate();logs.push(logging.logAttributeWarning(obj.fromAttribute))}}if(toNode&&hasFromAttribute){checkFileUpdate();let val=attribute.value.expression?{expression:_generateSpecCode(attribute.value.expression)}:attribute.value.value;_transferAttribute(component,toNode,attribute.name.name,val);logs.push(logging.logAttributeTransfer(component.name.name,toNode.name.name,attribute.name.name,val.expression?`{${val.expression}}`:val))}});if(hasDepAttribute&&!obj.fromAttribute&&obj.toAttribute&&!hasToAttribute){_appendAttribute(obj.toAttribute,obj.toValue,toNode?toNode:component);checkFileUpdate();logs.push(logging.logAttributeAddition(obj.toAttribute,obj.toValue,toNode?toNode.name.name:""))}}}return logs}function _migrateDescendantsHelper(children,componentMetadata,ast,code,newImports,childrenLogs,parent,migratedComp){for(let child of children){if(child.type==="JSXElement"&&child.openingElement.name&&child.openingElement.name.name===componentMetadata.legacyTag){if(!hasAttributesToIgnore(child,componentMetadata,parent)&&!hasExpressionAttributes(child.openingElement,componentMetadata.legacyTag,code,componentMetadata.importantAttributes)){if(componentMetadata["migrateTagByAttribute"]){childrenLogs.push(migrateTagByAttribute(child,componentMetadata["migrateTagByAttribute"],true))}else{childrenLogs.push(migrateTag(child,componentMetadata.legacyTag,componentMetadata.corePackTag,true))}for(let key in componentMetadata){if(typeof self[key]=="function"&&key!="migrateTagByAttribute"&&key!="migrateImports"){if(key==="migrateDescendants"){let childInfo=migrateDescendants(child,componentMetadata[key],code,ast,newImports,childrenLogs,parent);newImports=childInfo.imports;childrenLogs=childInfo.logs}else{let newLogs=self[key](child,componentMetadata[key],code,ast);if(newLogs&&newLogs.length>0){childrenLogs=childrenLogs.concat(newLogs)}}}}if(!migratedComp[componentMetadata.legacyTag]){migratedComp[componentMetadata.legacyTag]=1}else{migratedComp[componentMetadata.legacyTag]+=1}code=generateCode(ast);if(componentMetadata["migrateImports"]&&!inManifestData(componentMetadata.legacyTag)){newImports=[componentMetadata]}}}else{if(child.children){let childInfo=_migrateDescendantsHelper(child.children,componentMetadata,ast,code,newImports,childrenLogs,parent,migratedComp);childrenLogs=childInfo.logs;newImports=childInfo.imports;migratedComp=childInfo.migrated}}}return{imports:newImports,logs:childrenLogs,migrated:migratedComp}}function migrateDescendants(node,metadata,code,ast,newImports=[],childrenLogs=[]){let migratedComp={};const data=metadata.migrateDescendants;let returnValue=false;for(let metaName of data){let componentMetadata=require("../metadata/"+metaName+".json");if(node.children){returnValue=_migrateDescendantsHelper(node.children,componentMetadata,ast,code,newImports,childrenLogs,metadata.legacyTag,migratedComp);newImports=returnValue.imports;if(!returnValue){return false}}}return returnValue}function removeComponent(node){const tag=node.openingElement.name.name;const emptyNode={type:"JSXText",value:""};if(node.type==="JSXElement"){node.openingElement=emptyNode;node.closingElement=emptyNode}return logging.logComponentRemoved(tag)}function replaceComponent(fromNode,data){const tag=fromNode.openingElement.name.name;for(let obj of data){const descendant=_findDescendant(fromNode,obj.descendant);if(descendant){fromNode.openingElement=descendant.openingElement;fromNode.closingElement=descendant.closingElement;fromNode.children=descendant.children;return logging.logComponentReplacement(tag,descendant.openingElement.name.name)}}}exports.inManifestData=inManifestData;exports.generateAST=generateAST;exports.generateCode=generateCode;exports.hasAttributesToIgnore=hasAttributesToIgnore;exports.hasExpressionAttributes=hasExpressionAttributes;exports.appendImport=appendImport;exports.commentSlotContent=commentSlotContent;exports.slotContentToAttribute=slotContentToAttribute;exports.classNameToAttribute=classNameToAttribute;exports.handleSpecialCase=handleSpecialCase;exports.migrateImports=migrateImports;exports.migrateTag=migrateTag;exports.migrateTagByAttribute=migrateTagByAttribute;exports.migrateAttributes=migrateAttributes;exports.migrateDescendants=migrateDescendants;exports.checkFileUpdate=checkFileUpdate;exports.replaceComponent=replaceComponent;exports.removeComponent=removeComponent;