/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const traverse=require("ast-traverse");const logging=require("../utilities/logging");const generate=require("@babel/generator").default;const{parse}=require("@babel/parser");const astUtils=require("./astUtils");function generateAST(code){return parse(code,{sourceType:"unambiguous",plugins:global.plugins})}function generateCode(ast){return generate(ast,{retainLines:false}).code}function appendImports(ast,index,imports,corePackImport){if(corePackImport){for(let i of imports){if(!corePackImport.specifiers.some(s=>s.imported&&s.imported.name===i)){let newSpec={type:"JSXText",value:i};corePackImport.specifiers.push(newSpec);astUtils.checkFileUpdate();logging.logImportAppend(i)}}}else{let importMsg="import { ";for(let i in imports){if(i==0){importMsg+=imports[i]}else{importMsg+=", ";importMsg+=imports[i]}}importMsg+=" } from '@oracle/oraclejet-core-pack/webdriver';";let importNode={type:"JSXText",value:importMsg};ast.program.body.splice(index,0,importNode);astUtils.checkFileUpdate();logging.logImportAppend(importMsg)}}function hasMethodToIgnore(nodes,data,instances){if(data.ruleSetToIgnore){let result=false;let ruleSetToIgnore=data.ruleSetToIgnore;let type=data.type;let locator=data.locator;for(let method of ruleSetToIgnore){for(let n of nodes){if(n.type==="VariableDeclaration"){for(let d of n.declarations){if(d.init&&d.init.argument&&d.init.argument.callee&&d.init.argument.callee.name===locator.from){instances.push(d.id.name)}}}traverse(n,{pre:function(node){if(node.type==="MemberExpression"&&node.property.name===method&&instances.includes(node.object.name)){result=true;astUtils.checkFileUpdate();logging.logMethodDeprecation(method,type.to)}}})}}return result}}function classNameToAttribute(nodes,data,instances){let classInstances=[];for(let obj of data){for(let n of nodes){traverse(n,{pre:function(node){if(node.type==="CallExpression"){if(node.callee.object&&instances.includes(node.callee.object.name)&&node.callee.property.name==="getAttribute"&&node.arguments[0].value==="class"){if(n.type==="VariableDeclaration"){let addInstance=true;for(let instance of classInstances){if(instance.name===n.declarations[0].id.name){instance.parentIndex=nodes.indexOf(n);addInstance=false}}if(addInstance){classInstances.push({name:n.declarations[0].id.name,parentIndex:nodes.indexOf(n)})}}else if(n.type==="ExpressionStatement"&&n.expression.type==="AssignmentExpression"){for(let i in classInstances){if(classInstances[i].name===n.expression.left.name){classInstances[i].parentIndex=nodes.indexOf(n)}}}else if(n.type==="ExpressionStatement"&&n.expression.callee){if(n.expression.callee.object.object.callee.name==="expect"&&n.expression.arguments[0].type==="StringLiteral"&&n.expression.arguments[0].value===obj.className){node.arguments[0].value=obj.attribute;n.expression.arguments[0].value=obj.attributeValue;logging.logTestAttributeMigration("class",obj.attribute,obj.className,obj.attributeValue);astUtils.checkFileUpdate()}}}else if(classInstances.length>0&&node.callee.name==="expect"&&n.expression.arguments[0].type==="StringLiteral"&&n.expression.arguments[0].value===obj.className){for(let instance of classInstances){if(node.arguments[0].name===instance.name){if(nodes[instance.parentIndex].type==="VariableDeclaration"){nodes[instance.parentIndex].declarations[0].init.argument.arguments[0].value=obj.attribute}else{nodes[instance.parentIndex].expression.right.argument.arguments[0].value=obj.attribute}n.expression.arguments[0].value=obj.attributeValue;logging.logTestAttributeMigration("class",obj.attribute,obj.className,obj.attributeValue);astUtils.checkFileUpdate();break}}}}}})}}}function migrateAttributes(nodes,data,instances,webElement){for(let obj of data){let attInstances=[];for(let n of nodes){traverse(n,{pre:function(node){if(node.type==="CallExpression"){if(node.callee.object&&instances.includes(node.callee.object.name)&&node.callee.property.name==="getAttribute"&&node.arguments[0].value===obj.fromAttribute){if(n.type==="VariableDeclaration"){if(!obj.toAttribute){logging.logTestAttributeDeprecation(obj.fromAttribute,obj.fromValue,webElement)}else{if(!obj.fromValue){node.arguments[0].value=obj.toAttribute;astUtils.checkFileUpdate();logging.logTestAttributeMigration(obj.fromAttribute,obj.toAttribute,obj.fromValue,obj.toValue)}else{let addInstance=true;for(let i in attInstances){if(attInstances[i].name===n.declarations[0].id.name){attInstances[i].parentIndex=nodes.indexOf(n);addInstance=false}}if(addInstance){attInstances.push({name:n.declarations[0].id.name,parentIndex:nodes.indexOf(n)})}}}}else if(n.type==="ExpressionStatement"&&n.expression.type==="AssignmentExpression"){if(!obj.toAttribute){logging.logTestAttributeDeprecation(obj.fromAttribute,obj.fromValue,webElement)}else{if(!obj.toValue){n.expression.right.argument.arguments[0].value=obj.toAttribute;astUtils.checkFileUpdate();logging.logTestAttributeMigration(obj.fromAttribute,obj.toAttribute,obj.fromValue,obj.toValue)}else{for(let i in attInstances){if(attInstances[i].name===n.expression.left.name){attInstances[i].parentIndex=nodes.indexOf(n)}}}}}else if(n.type==="ExpressionStatement"&&n.expression.callee&&n.expression.callee.object.object.callee.name==="expect"){if(!obj.toAttribute){astUtils.checkFileUpdate();logging.logTestAttributeDeprecation(obj.fromAttribute,obj.fromValue,webElement)}else if(obj.fromValue&&n.expression.arguments[0].value===obj.fromValue||!obj.fromValue){node.arguments[0].value=obj.toAttribute;if(obj.fromValue){n.expression.arguments[0].value=obj.toValue?obj.toValue:n.expression.arguments[0].value}astUtils.checkFileUpdate();logging.logTestAttributeMigration(obj.fromAttribute,obj.toAttribute,obj.fromValue,obj.toValue)}}}else if(attInstances.length>0&&node.callee.name==="expect"){for(let instance of attInstances){if(node.arguments[0].name===instance.name){if(obj.fromValue&&n.expression.arguments[0].value===obj.fromValue){n.expression.arguments[0].value=obj.toValue;if(obj.fromAttribute!=obj.toAttribute){nodes[instance.parentIndex].declarations[0].init.argument.arguments[0].value=obj.toAttribute}astUtils.checkFileUpdate();logging.logTestAttributeMigration(obj.fromAttribute,obj.toAttribute,obj.fromValue,obj.toValue)}}}}}}})}}}function migrateLocator(nodes,data,instances){for(let node of nodes){traverse(node,{pre:function(n){if(n.type==="VariableDeclaration"){for(let d of n.declarations){let declaration;if(d.init&&d.init.argument&&d.init.argument.callee){declaration=d.init.argument.callee}else if(d.init&&d.init.argument){let arg=d.init.argument;while(!arg.callee&&arg.argument){arg=arg.argument}declaration=arg.callee}if(declaration&&declaration.name===data.from){instances.push(d.id.name);declaration.name=data.to;let variable=d.id.name;astUtils.checkFileUpdate();logging.logLocatorMigration(data.from,data.to,variable)}}}else if(n.type==="ExpressionStatement"&&n.expression.type==="AssignmentExpression"&&n.expression.right.argument){let arg=n.expression.right.argument;while(!arg.callee&&arg.argument){arg=arg.argument}if(arg.callee&&arg.callee.name===data.from){arg.callee.name=data.to}}else if(n.type==="CallExpression"&&n.callee.name===data.from){n.callee.name=data.to}}})}return instances}function migrateCssLoc(nodes,componentMetadata){for(let n of nodes){traverse(n,{pre:function(node){if(node.type==="CallExpression"&&node.callee.object&&node.callee.object.name==="By"&&(node.callee.property.name==="css"||node.callee.property.name==="tagName")){for(let arg of node.arguments){let newVal=searchAndReplace(arg.value,componentMetadata.legacyTag,componentMetadata.corePackTag);if(newVal){arg.value=newVal;astUtils.checkFileUpdate();logging.logLocatorMigration(componentMetadata.legacyTag,componentMetadata.corePackTag)}}}}})}}function migrateType(node,data,globalInstances){if(node.id){globalInstances.push(node.id.name);node.id.typeAnnotation.typeAnnotation.typeName.name=data.to;astUtils.checkFileUpdate();logging.logTypeMigration(data.from,data.to)}else{node.object.name=data.to}return globalInstances}function migrateImports(node,data,imports){let toRemove=[];for(let spec of node.specifiers){if(spec.imported.name===data.locator.from){imports.push(data.locator.to);toRemove.push(spec)}else if(spec.imported.name===data.type.from){imports.push(data.type.to);toRemove.push(spec)}}for(let spec of toRemove){node.specifiers.splice(node.specifiers.indexOf(spec),1)}if(node.specifiers.length===0){node.type="JSXText";node.value=""}return imports}function searchAndReplace(code,from,to){from=from.replace(/[.*+?^${}()|[\]\\]/g,"\\$&");regex=new RegExp("\\b"+from+"\\b(?![\\w-])","g");if(regex.test(code)){return code.replace(regex,to)}else{return false}}exports.generateAST=generateAST;exports.generateCode=generateCode;exports.migrateType=migrateType;exports.migrateLocator=migrateLocator;exports.migrateCssLoc=migrateCssLoc;exports.migrateImports=migrateImports;exports.appendImports=appendImports;exports.migrateAttributes=migrateAttributes;exports.hasMethodToIgnore=hasMethodToIgnore;exports.classNameToAttribute=classNameToAttribute;