/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const path=require("path");const Utils=require("../jetwc-lib/jetwcutils-componentAuditUtils");const ruleDef=function(){};ruleDef.prototype.getName=function(){return"jetwc-require-paths"};ruleDef.prototype.getShortDescription=function(){return"Checks that any import paths used are valid"};ruleDef.prototype.getDescription=function(){return"This rule ensures that all path references (define, require or import) use paths that are valid in the context of the component. This list of valid paths will be constructed based on the declared and implicit dependencies of the component, combined with the standardPaths configuration property of the jetwc-require-paths rule which you can add to if needs be. The path \".\" is always valid. The path .. should not be used outside of the boundary of a given component, use the pack root instead."};ruleDef.prototype.register=function(context){return{CallExpression:this._doAuditRequireStyle,ImportDeclaration:this._doAuditImport,TSImportEqualsDeclaration:this._doAuditImportRequire}};ruleDef.prototype._doAuditRequireStyle=function(context,node){const NT=context.utils.AstUtils.getNodeTypes();if(context.rulePack.getExtension()&&context.rulePack.getExtension().hasOwnProperty("allComponents")){const withinComponent=Utils.matchComponentPath(context,path.dirname(context.filepath));const componentInfo=context.rulePack.getExtension().allComponents.get(withinComponent);if(withinComponent&&node.callee.type===NT.IDENTIFIER&&(node.callee.name==="define"||node.callee.name==="require")){if(node.arguments[0].type===NT.ARRAY_EXPR&&node.arguments[0].elements.length>0){node.arguments[0].elements.forEach(arg=>{if(arg.value){this._verifyPath(context,arg.value,componentInfo,node.callee.name)}})}else if(node.arguments[0].type===NT.LITERAL&&node.arguments[0].value){this._verifyPath(context,node.arguments[0].value,componentInfo,node.callee.name)}}}else{Utils.reportMissingBootstrapInfo(context,"allComponents")}};ruleDef.prototype._doAuditImport=function(context,node){if(context.rulePack.getExtension()&&context.rulePack.getExtension().hasOwnProperty("allComponents")){const NT=context.utils.AstUtils.getNodeTypes();const withinComponent=Utils.matchComponentPath(context,path.dirname(context.filepath));const componentInfo=context.rulePack.getExtension().allComponents.get(withinComponent);if(componentInfo){if(node.source&&node.source.type===NT.LITERAL){const sourcePath=node.source.value;this._verifyPath(context,sourcePath,componentInfo,"import")}}}else{Utils.reportMissingBootstrapInfo(context,"allComponents")}};ruleDef.prototype._doAuditImportRequire=function(context,node){const NT=context.utils.AstUtils.getNodeTypes();if(context.rulePack.getExtension()&&context.rulePack.getExtension().hasOwnProperty("allComponents")){const withinComponent=Utils.matchComponentPath(context,path.dirname(context.filepath));const componentInfo=context.rulePack.getExtension().allComponents.get(withinComponent);if(componentInfo){if(node.moduleReference&&node.moduleReference.type===NT.EXTERN_MOD_REF&&node.moduleReference.expression){const sourcePath=node.moduleReference.expression.value;this._verifyPath(context,sourcePath,componentInfo,"importRequire")}}}else{Utils.reportMissingBootstrapInfo(context,"allComponents")}};ruleDef.prototype._verifyPath=function(context,pathToCheck,componentInfo,importType){if(pathToCheck&&typeof pathToCheck==="string"){let pathVerified=false;const allValidPaths=context.rulePack.getRuleCustomOption("standardPaths")?.slice(0);const additionalPaths=context.rulePack.getRuleCustomOption("additionalPaths");if(additionalPaths&&Array.isArray(additionalPaths)){additionalPaths.forEach(addPath=>{allValidPaths.push(addPath)})}if(componentInfo.requirePaths){componentInfo.requirePaths.forEach(depPath=>{allValidPaths.push(depPath)})}let targetPath=pathToCheck;const pluginPos=targetPath.indexOf("!");if(pluginPos>=0){targetPath=targetPath.substring(pluginPos+1)}const targetPathParts=targetPath.split("/");if(targetPathParts[0]!==".."){if(allValidPaths.includes(targetPathParts[0])){pathVerified=true}else{if(targetPathParts[1]){const packCompPath=targetPathParts[0]+"/"+targetPathParts[1];if(allValidPaths.includes(packCompPath)){pathVerified=true}}}}else{let walkBackDepth=0;let examineSegment="..";while(examineSegment===".."){examineSegment=targetPathParts.shift();if(examineSegment===".."){walkBackDepth++}}const compRootPathParts=componentInfo.rootFolder.split("/");const importingFileParts=context.filepath.split("/");for(let i=walkBackDepth;i>=0;i--){importingFileParts.pop()}if(importingFileParts.length>=compRootPathParts.length){if(importingFileParts.slice(0,compRootPathParts.length).join("/")===componentInfo.rootFolder){pathVerified=true}}}if(!pathVerified){let sourceInfo;switch(importType){case"import":sourceInfo="Import";break;case"importRequire":sourceInfo="Import via require()";break;case"define":sourceInfo="Import via define()";break;case"require":sourceInfo="Import via require()";break};issue=new context.Issue(`${sourceInfo} for component ${componentInfo.fullName} is not based on a valid path in that component context. Valid paths would be [${allValidPaths.join(",")}] encountered: '${pathToCheck}'`);context.reporter.addIssue(issue,context)}}};module.exports=ruleDef;