/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const path=require("path");const Utils=require("../jetwc-lib/jetwcutils-componentAuditUtils");const ruleDef=function(){};ruleDef.prototype.getName=function(){return"jetwc-writeback-required"};ruleDef.prototype.getShortDescription=function(){return"Verifies that properties are marked with writeback if needed"};ruleDef.prototype.getDescription=function(){return"This rule attempts to match up calls to the setProperty() api, or direct assignments to this.properties, with the properties being addressed and then verifies that they are set to writeback:true.  (The properties object should not be used as a private cache for component state)"};ruleDef.prototype.register=function(context){return{CallExpression:this._doAuditOnSetProperty,AssignmentExpression:this._doAuditOnPropertyAssignment,attrexpr:this._doAuditOnPropertyHTMLRef}};ruleDef.prototype._doAuditOnSetProperty=function(context,node){let issue;const NT=context.utils.AstUtils.getNodeTypes();const withinComponent=Utils.matchComponentPath(context,path.dirname(context.filepath));if(withinComponent&&node.callee.property&&node.callee.property.type===NT.IDENTIFIER&&node.callee.property.name==="setProperty"){let isStyle=false;if(node.callee.object&&node.callee.object.property&&node.callee.object.property.type===NT.IDENTIFIER&&node.callee.object.property.name==="style"){isStyle=true}let compRef=false;if(node.callee.object&&node.callee.object.object&&node.callee.object.property&&node.callee.object.property.type===NT.IDENTIFIER){if(node.callee.object.object.type===NT.THIS_EXPR||node.callee.object.object.type===NT.IDENTIFIER&&node.callee.object.object.name==="self"){compRef=true}}if(!isStyle&&compRef&&node.arguments&&node.arguments.length===2){if(node.arguments[0].type=NT.LITERAL&&node.arguments[0].value){let potentialProperty=node.arguments[0].value;if(potentialProperty.includes(".")){potentialProperty=potentialProperty.split(".")[0]}const componentInfo=Utils.componentInfo(context,withinComponent);if(componentInfo.properties&&componentInfo.properties.hasOwnProperty(potentialProperty)){if(!componentInfo.properties[potentialProperty].hasOwnProperty("writeback")||!componentInfo.properties[potentialProperty].writeback){issue=new context.Issue(`Component '${withinComponent}' makes a call to setProperty() for the property ${potentialProperty} but this is not marked as writeback in the metadata`);issue.setMsgKey(context.ruleName+"_1");context.reporter.addIssue(issue,context,"blocker")}}else{this._reportUndclProp(context,withinComponent,potentialProperty)}}}}};ruleDef.prototype._doAuditOnPropertyAssignment=function(context,node){let issue;const NT=context.utils.AstUtils.getNodeTypes();const withinComponent=Utils.matchComponentPath(context,path.dirname(context.filepath));if(withinComponent&&node.left.type===NT.MEMBER_EXPR&&node.left.object.type===NT.MEMBER_EXPR&&node.left.object.property.name==="properties"){const setProperty=this._isContextProperties(context,node);if(setProperty){const componentInfo=Utils.componentInfo(context,withinComponent);if(componentInfo.properties&&componentInfo.properties.hasOwnProperty(setProperty)){if(!componentInfo.properties[setProperty].hasOwnProperty("writeback")||!componentInfo.properties[setProperty].writeback){issue=new context.Issue(`Component '${withinComponent}' assigns a value to the property ${setProperty} but this is not marked as writeback in the metadata`);issue.setMsgKey(context.ruleName+"_2");context.reporter.addIssue(issue,context)}}else{this._reportUndclProp(context,withinComponent,setProperty)}}}};ruleDef.prototype._doAuditOnPropertyHTMLRef=function(context,attribName,attribValue,rawAttribValue){let issue;const withinComponent=Utils.matchComponentPath(context,path.dirname(context.filepath));if(withinComponent){const parser=/^(?:\{\{\$props\.|\{\{\$properties\.)(.*)(?:\}\})$/;if(attribValue.match(parser)){const expressionParts=attribValue.split(parser);const checkProperty=expressionParts[1];const componentInfo=Utils.componentInfo(context,withinComponent);if(componentInfo.properties&&componentInfo.properties.hasOwnProperty(checkProperty)){if(!componentInfo.properties[checkProperty].hasOwnProperty("writeback")||!componentInfo.properties[checkProperty].writeback){issue=new context.Issue(`Component '${withinComponent}' assigns the property '${checkProperty}' using a read-write binding to the <${context.tag}> attribute '${attribName}', however, the property is not marked as writeback in the metadata`);issue.setMsgKey(context.ruleName+"_3");context.reporter.addIssue(issue,context)}}else{this._reportUndclProp(context,withinComponent,checkProperty)}}}};ruleDef.prototype._reportUndclProp=function(context,component,property){issue=new context.Issue(`Component '${component}' makes a call to setProperty() or cached context.properties for an undeclared property ${property}`);issue.setMsgKey(context.ruleName+"_4");context.reporter.addIssue(issue,context,"blocker")};ruleDef.prototype._isContextProperties=function(context,node){let assignedProperty;let assignmentChain=Utils.decomposeAssignment(context,node);const assignmentRoot=assignmentChain.pop();const assignmentSecondary=assignmentChain.pop();if(assignmentRoot==="this"||assignmentRoot==="<parameter>"&&assignmentSecondary==="context"){assignedProperty=assignmentChain[0]}return assignedProperty};module.exports=ruleDef;