/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const Utils=require("../jetwc-lib/jetwcutils-componentAuditUtils");const AuditComponentCacheManager=require("../jetwc-lib/jetwcutils-auditComponentCacheManager");const NetUtils=require("../../lib/NetUtils");const CommonImpl=require("../jetwc-lib/jetwcutils-commonListenerImpls");const TsxUtils=require("../jetwc-lib/jetwcutils-tsxUtils");const ruleDef=function(){};ruleDef.prototype.getName=function(){return"jetwco-component-api-baseline-changes"};ruleDef.prototype.getShortDescription=function(){return"Check components for API changes against a base version"};ruleDef.prototype.getDescription=function(){return"This audit compares the APIs exposed by components with a designated baseline version of a component in the Exchange Catalog and checks if the component APIs have changed"};ruleDef.prototype.register=function(context){return{startaudit:this._auditPrep,closedownRP:this._dealWithCache,compjson:this._doAudit,Program:this._doVComponentAudit}};ruleDef.prototype._auditPrep=function(context){let baselineConfigured=false;let baselineMetadata=[];this.exchangeURL=CommonImpl.checkForExchangeConfiguration(context);if(this.exchangeURL){this.acCache=new AuditComponentCacheManager(context,this.getName());if(this.acCache.isBaselineIndexAvailable()){const exchangeRoot=this.exchangeURL;let baselineIndex=this.acCache.readComponentCacheIndexFile();if(Object.entries(baselineIndex).length===0){const baselineConfiguration=this.acCache.getBaselineOpts();let exchangeReadError=false;for(const[baselineComponent,baselineComponentVersion]of Object.entries(baselineConfiguration)){const eachComponentInfo=this._cacheBaselineMetadataForComponent(context,baselineComponent,baselineComponentVersion,exchangeRoot);if(!eachComponentInfo.inError){baselineMetadata=baselineMetadata.concat(eachComponentInfo.metadata)}else{exchangeReadError=true}}if(!exchangeReadError){baselineMetadata.forEach(component=>{const fullName=Utils.getComponentFullName(component);baselineIndex[fullName]=component.version;if(!this.acCache.existsInComponentCache(fullName,component.version)){this.acCache.writeToComponentCache(fullName,component.version,component)}});this.acCache.writeComponentCacheIndexFile(baselineIndex)}}if(Object.entries(baselineIndex).length>0){baselineConfigured=true;if(Object.keys(baselineIndex).length>0){Utils.safeSetExtension(context,{baselineIndex:baselineIndex})}}}}};ruleDef.prototype._cacheBaselineMetadataForComponent=function(auditContext,componentName,componentVersion,exchangeUrl){const msg=auditContext.utils.msgLib;let componentMetaArray=[];let inErrorCondition=false;try{let componentMeta;if(this.acCache.existsInComponentCache(componentName,componentVersion)){componentMeta=this.acCache.readFromComponentCache(componentName,componentVersion)}else{const netUtils=new NetUtils;const componentMetaResp=netUtils.loadUrl(`${exchangeUrl}/components/${componentName}/versions/${componentVersion}`);if(componentMetaResp.statusCode===200&&componentMetaResp.body){const componentResponseMeta=JSON.parse(componentMetaResp.body);componentMeta=componentResponseMeta.component}else if(componentMetaResp.statusCode===403){msg.info(`${auditContext.ruleName} - Baseline component ${componentName}@${componentVersion} cannot be read - skipping`)}else{const issue=new auditContext.Issue(`Unexpected error for the rule ${auditContext.ruleName} reading ${componentName}@${componentVersion}: Response ${componentMetaResp.statusCode} (${exchangeUrl})`);issue.setMsgKey("jetwco-component-api-baseline-changes_2");auditContext.reporter.addIssue(issue,auditContext,"critical");inErrorCondition=true}}if(!inErrorCondition&&componentMeta){const compType=componentMeta.type?componentMeta.type:"composite";if(compType==="pack"){const dependents=componentMeta.dependencies;if(dependents){for(const[packComponent,packComponentVersion]of Object.entries(dependents)){const eachPackMemberInfo=this._cacheBaselineMetadataForComponent(auditContext,packComponent,packComponentVersion,exchangeUrl);if(!eachPackMemberInfo.inError){componentMetaArray=componentMetaArray.concat(eachPackMemberInfo.metadata)}else{inErrorCondition=true}}}}componentMetaArray.push(componentMeta)}}catch(e){msg.error(`Exchange read error ${JSON.stringify(e)}`);const issue=new auditContext.Issue(`The exchangeURL value specified for the rule ${auditContext.ruleName} is incorrect: ${exchangeUrl}`);issue.setMsgKey("jetwco-component-api-baseline-changes_2");auditContext.reporter.addIssue(issue,context,"critical");inErrorCondition=true};return{inError:inErrorCondition,metadata:componentMetaArray}};ruleDef.prototype._dealWithCache=function(context){const shouldClearCache=context.rulePack.getRuleCustomOption("clearCache",this.getName());if(shouldClearCache&&this.acCache){this.acCache.removeLocalCache()}};ruleDef.prototype._doAudit=function(context){if(context.suppData.obj){const meta=context.suppData.obj;const fullName=Utils.getComponentFullName(meta);const componentType=Utils.getComponentType(meta);this._baselineCheck(context,fullName,componentType,meta)}};ruleDef.prototype._doVComponentAudit=function(context){const componentInfo=TsxUtils.matchToVComponentImpl(context,context.filepath);if(componentInfo){let remappedDependencies;if(componentInfo.dependencies){remappedDependencies=componentInfo.dependencies.reduce((asMap,entry)=>{asMap[entry.name]=entry.range;return asMap},{})}const meta={version:componentInfo.version,jetVersion:componentInfo.jetVersion,properties:componentInfo.properties,methods:componentInfo.methods,events:componentInfo.events,sots:componentInfo.slots,dependencies:remappedDependencies};this._baselineCheck(context,componentInfo.fullName,componentInfo.type,meta)}};ruleDef.prototype._baselineCheck=function(context,fullName,componentType,componentMeta){const baselineIndex=context.rulePack.getExtension().baselineIndex;if(componentMeta&&baselineIndex){const componentVersion=componentMeta.version;const baselineVersion=baselineIndex[fullName];if(baselineVersion){const semver=context.utils.semVerUtils;const baselineComponentMeta=this.acCache.readFromComponentCache(fullName,baselineVersion);try{switch(componentType){case"pack":this._conductCheck(this._packContentsDiff,componentMeta,baselineComponentMeta);this._conductCheck(this._packBundlesDiff,componentMeta,baselineComponentMeta);break;case"composite":this._conductCheck(this._detectJetVersionDiff,semver,componentMeta,baselineComponentMeta);this._conductCheck(this._detectDependencyDiff,semver,componentMeta,baselineComponentMeta);this._conductCheck(this._detectMethodDiff,componentMeta,baselineComponentMeta);this._conductCheck(this._detectEventDiff,componentMeta,baselineComponentMeta);this._conductCheck(this._detectSlotDiff,componentMeta,baselineComponentMeta);this._conductCheck(this._detectDynamicSlotDiff,componentMeta,baselineComponentMeta);this._conductCheck(this._detectPropertyDiff,componentMeta,baselineComponentMeta);break;case"vbcs-pattern":this._conductCheck(this._detectDependencyDiff,semver,componentMeta,baselineComponentMeta);break;case"resource":this._conductCheck(this._detectJetVersionDiff,semver,componentMeta,baselineComponentMeta);this._conductCheck(this._detectDependencyDiff,semver,componentMeta,baselineComponentMeta);this._conductCheck(this._detectPublicModulesDiff,componentMeta,baselineComponentMeta);break}}catch(auditDetail){const issue=new context.Issue(`API change for ${fullName}@${componentVersion} detected in comparison with baseline version ${baselineVersion}. Indication: ${auditDetail}`);issue.setMsgKey("jetwco-component-api-baseline-changes_1");context.reporter.addIssue(issue,context)}}}};ruleDef.prototype._conductCheck=function(checkCallback,semver,config,existingConfig){const result=checkCallback.call(this,semver,config,existingConfig);if(result){throw result}};ruleDef.prototype._detectJetVersionDiff=function(semver,config,existingConfig,claimedMinorDiff){let explanation;const componentJetVersion=config.jetVersion?config.jetVersion:"0.0.0";const existingJetVersion=existingConfig.jetVersion?existingConfig.jetVersion:"0.0.0";if(componentJetVersion!==existingJetVersion){if(existingJetVersion==="0.0.0"||componentJetVersion==="0.0.0"){explanation="JET version range has been added or removed"}else{const minVersion=semver.minVersion(componentJetVersion);const minExistingVersion=semver.minVersion(existingJetVersion);if(semver.major(minVersion)>semver.major(minExistingVersion)){explanation="minimum major JET version required has increased"}else if(semver.minor(minVersion)>semver.minor(minExistingVersion)){explanation="minimum minor JET version required has increased"}}}return explanation};ruleDef.prototype._packContentsDiff=function(config,existingConfig){let explanation;const configDeps=config.dependencies?Object.keys(config.dependencies):[];const existingDeps=existingConfig.dependencies?Object.keys(existingConfig.dependencies):[];if(configDeps.length>0){for(let i=0;i<existingDeps.length;i++){const key=existingDeps[i];if(!config.dependencies.hasOwnProperty(key)){explanation=`Pack component ${key} has been removed`;break}}}return explanation};ruleDef.prototype._packBundlesDiff=function(config,existingConfig){let explanation;const configBundles=config.bundles?Object.keys(config.bundles):[];const existingBundles=existingConfig.bundles?Object.keys(existingConfig.bundles):[];for(let i=0;i<existingBundles.length;i++){const key=existingBundles[i];if(!config.bundles.hasOwnProperty(key)){explanation=`Pack bundle '${key}' has been removed`;break}else{const configContents=config.bundles[key];const existingContents=existingConfig.bundles[key];for(let i=0;i<existingContents.length;i++){const module=existingContents[i];if(!configContents.includes(module)){explanation=`Pack bundle '${key}' entry '${module}' removed`;break}}}}return explanation};ruleDef.prototype._detectPublicModulesDiff=function(config,existingConfig){let explanation;const configPM=config.publicModules?config.publicModules:[];const existingPM=existingConfig.publicModules?existingConfig.publicModules:[];for(let i=0;i<existingPM.length;i++){const module=existingPM[i];if(!configPM.includes(module)){explanation=`Public Module entry '${module}' has been removed`;break}}return explanation};ruleDef.prototype._detectDependencyDiff=function(semver,config,existingConfig){let explanation;const configDeps=config.dependencies?Object.keys(config.dependencies):[];const existingDeps=existingConfig.dependencies?Object.keys(existingConfig.dependencies):[];if(configDeps.length>existingDeps.length){explanation="More dependencies are declared by component"}else{if(configDeps.length>0){for(let i=0;i<existingDeps.length;i++){const key=existingDeps[i];if(config.dependencies.hasOwnProperty(key)){const newDepVersion=semver.minVersion(config.dependencies[key]);const existingDepVersion=semver.minVersion(existingConfig.dependencies[key]);if(semver.major(newDepVersion)!==semver.major(existingDepVersion)){explanation="Different major version of dependency "+key;break}if(semver.minor(newDepVersion)!==semver.minor(existingDepVersion)){explanation="Different minor version of dependency "+key;break}}else{explanation=`Dependency ${key} has been removed`;break}}}}return explanation};ruleDef.prototype._detectMethodDiff=function(config,existingConfig){let explanation;const configMethods=config.methods?Object.keys(config.methods):[];const existingMethods=existingConfig.methods?Object.keys(existingConfig.methods):[];for(let i=0;i<existingMethods.length;i++){const key=existingMethods[i];if(config.methods.hasOwnProperty(key)){const configMethodParams=config.methods[key].params?config.methods[key].params:[];const existingMethodParams=existingConfig.methods[key].params?existingConfig.methods[key].params:[];for(let j=0;j<existingMethodParams.length;j++){if(j>=configMethodParams.length){explanation=`Existing method '${key}' has fewer parameters`;break}else{if(!this._compareType(configMethodParams[j],existingMethodParams[j])){explanation=`Method '${key}' parameter number:${j+1} has a changed type`;break}}}if(!explanation&&configMethodParams.length>existingMethodParams.length){explanation=`Existing method '${key}' has additional parameters`;break}}else{explanation=`Existing method '${key}' has been removed`;break}}return explanation};ruleDef.prototype._detectPropertyDiff=function(config,existingConfig){let explanation;const configProps=config.properties?Object.keys(config.properties):[];const existingProps=existingConfig.properties?Object.keys(existingConfig.properties):[];explanation=this._compareProperties(config.properties,existingConfig.properties);if(!explanation){const addedRequiredProps=configProps.filter(addedProp=>!existingProps.includes(addedProp)&&config[addedProp]&&config[addedProp].required);if(addedRequiredProps.length>0){explanation=`Required property(ies) added: ${addedRequiredProps.join(",")}`}}return explanation};ruleDef.prototype._detectEventDiff=function(config,existingConfig){let explanation;const configEvents=config.events?Object.keys(config.events):[];const existingEvents=existingConfig.events?Object.keys(existingConfig.events):[];for(let i=0;i<existingEvents.length;i++){const key=existingEvents[i];if(config.events.hasOwnProperty(key)){const configEventDetail=config.events[key].detail?config.events[key].detail:{};const existingEventDetail=existingConfig.events[key].detail?existingConfig.events[key].detail:{};const detailKeys=Object.keys(configEventDetail);const existingDetailKeys=Object.keys(existingEventDetail);for(let j=0;j<existingDetailKeys.length;j++){const detailKey=existingDetailKeys[j];if(j>=detailKeys.length||!configEventDetail.hasOwnProperty(detailKey)){explanation=`Event '${key}' has missing entries in the detail payload`;break}else{if(!this._compareType(configEventDetail[detailKey],existingEventDetail[detailKey])){explanation=`Event '${key}' detail attribute '${detailKey}' has a changed type`;break}}}if(!explanation&&config.events[key].bubbles!==existingConfig.events[key].bubbles){explanation=`Event '${key}' has a changed bubbling state`;break}if(!explanation&&config.events[key].cancelable!==existingConfig.events[key].cancelable){explanation=`Event '${key}' has changed cancelable state`;break}}else{explanation=`Event '${key}' has been removed`;break}}return explanation};ruleDef.prototype._detectSlotDiff=function(config,existingConfig){return this._detectTypedSlotDiff("slots",config,existingConfig)};ruleDef.prototype._detectDynamicSlotDiff=function(config,existingConfig){return this._detectTypedSlotDiff("dynamicSlots",config,existingConfig)};ruleDef.prototype._detectTypedSlotDiff=function(slotType,config,existingConfig){let explanation;const configSlots=config[slotType]?Object.keys(config[slotType]):[];const existingSlots=existingConfig[slotType]?Object.keys(existingConfig[slotType]):[];for(let i=0;i<existingSlots.length;i++){const key=existingSlots[i];if(config[slotType].hasOwnProperty(key)){const configSlotData=config[slotType][key].data?config[slotType][key].data:{};const existingSlotData=existingConfig[slotType][key].data?existingConfig[slotType][key].data:{};const dataKeys=Object.keys(configSlotData);const existingDataKeys=Object.keys(existingSlotData);for(let j=0;j<existingDataKeys.length;j++){const dataKey=existingDataKeys[j];if(j>=dataKeys.length||!configSlotData.hasOwnProperty(dataKey)){explanation=`Template slot '${key}' - '${dataKey}' has been removed from data definition`;break}else{if(!this._compareType(configSlotData[dataKey],existingSlotData[dataKey])){explanation=`Template slot '${key}' data attribute '${dataKey}' has a changed type`;break}}}}else{explanation=`Slot '${key}' has been removed`;break}}return explanation};ruleDef.prototype._compareType=function(newTyped,existingTyped){let matches=true;const newType=newTyped&&newTyped.type?newTyped.type:null;const existingType=existingTyped&&existingTyped.type?existingTyped.type:null;if(newType!==existingType){if(newType.includes("|")){const newTypeOptions=newType?newType.split("|"):[];const existingTypeOptions=existingType?existingType.split("|"):[];matches=existingTypeOptions.every(checkType=>newTypeOptions.includes(checkType))}else{matches=false}}return matches};ruleDef.prototype._compareProperties=function(newProps,oldProps,path){let explanation;const configProps=newProps?Object.keys(newProps):[];const existingProps=oldProps?Object.keys(oldProps):[];for(let i=0;i<existingProps.length;i++){const key=existingProps[i];if(newProps.hasOwnProperty(key)){const newProperty=newProps[key];const oldProperty=oldProps[key];if(!this._compareType(newProperty,oldProperty)){explanation=`Property type change for '${path?path:""}${key}'`;break}if(oldProperty.hasOwnProperty("enumValues")){const newOptions=newProperty.hasOwnProperty("enumValues")?newProperty.enumValues:[];if(!oldProperty.enumValues.every(checkOption=>newOptions.includes(checkOption))){explanation=`Enumeration value option removed from property '${path?path:""}${key}'`;break}}else if(newProperty.hasOwnProperty("enumValues")){explanation=`Property '${path?path:""}${key}' is now constrained by an enumeration`;break}if(!path){if(oldProperty.writeback&&!newProperty.writeback){explanation=`Property '${key}' is no longer writeback enabled`;break}if(!oldProperty.required&&newProperty.required){explanation=`Property '${key}' has been made required`;break}const oldROState=oldProperty.hasOwnProperty("readOnly")?oldProperty.readOnly:false;const newROState=newProperty.hasOwnProperty("readOnly")?newProperty.readOnly:false;if(oldROState!==newROState){explanation=`Property '${key}' readOnly state has changed`;break}const oldDefault=oldProperty.hasOwnProperty("value")?JSON.stringify(oldProperty.value):"";const newDefault=newProperty.hasOwnProperty("value")?JSON.stringify(newProperty.value):"";if(oldDefault!==newDefault){explanation=`Property '${key}' default value has changed`;break}}if(newProperty.hasOwnProperty("properties")||oldProperty.hasOwnProperty("properties")){const newSubProps=newProperty.properties?newProperty.properties:{};const oldSubProps=oldProperty.properties?oldProperty.properties:{};const propPath=path?path+key+".":key+".";explanation=this._compareProperties(newSubProps,oldSubProps,propPath);if(explanation){break}}}else{explanation=`Property '${path?path:""}${key}' has been removed`;break}}return explanation};module.exports=ruleDef;