/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const Utils=require("../jetwc-lib/jetwcutils-componentAuditUtils");const AuditComponentCacheManager=require("../jetwc-lib/jetwcutils-auditComponentCacheManager");const NetUtils=require("../../lib/NetUtils");const CommonImpl=require("../jetwc-lib/jetwcutils-commonListenerImpls");const TsxUtils=require("../jetwc-lib/jetwcutils-tsxUtils");const VC=require("./jetwco-cacc-auditConstraints");const CaccHelpers=require("./jetwco-cacc-helpers");const{applyCustomMappings}=require("./jetwco-cacc-typeMappingOverrides");const jetVer=require("./jetwco-cacc-jetVer");const packContent=require("./jetwco-cacc-packContent");const packBundle=require("./jetwco-cacc-packBundle");const packBundleContent=require("./jetwco-cacc-packBundleContent");const dependencies=require("./jetwco-cacc-dependencies");const properties=require("./jetwco-cacc-properties");const propertyDefault=require("./jetwco-cacc-propertyDefault");const propertyRequired=require("./jetwco-cacc-propertyRequired");const propertyReadOnly=require("./jetwco-cacc-propertyReadOnly");const propertyWriteback=require("./jetwco-cacc-propertyWriteback");const propertyType=require("./jetwco-cacc-propertyType");const propertyEnumeration=require("./jetwco-cacc-propertyEnumeration");const methods=require("./jetwco-cacc-methods");const methodParams=require("./jetwco-cacc-methodParams");const methodParamTypes=require("./jetwco-cacc-methodParamTypes");const methodReturnType=require("./jetwco-cacc-methodReturnType");const events=require("./jetwco-cacc-events");const eventPayload=require("./jetwco-cacc-eventPayload");const eventBubbles=require("./jetwco-cacc-eventBubbles");const eventCancelable=require("./jetwco-cacc-eventCancelable");const slots=require("./jetwco-cacc-slots");const slotDataVariable=require("./jetwco-cacc-slotDataVariable");const slotDataVariableType=require("./jetwco-cacc-slotDataVariableType");const dynamicSlots=require("./jetwco-cacc-dynamicSlots");const styleVariables=require("./jetwco-cacc-styleVariables");const styleClasses=require("./jetwco-cacc-styleClasses");const publicModules=require("./jetwco-cacc-publicModules");const componentType=require("./jetwco-cacc-componentType");const CONTROL_PROFILE={jetVer:{callback:jetVer,add:VC.MAJOR,remove:VC.MAJOR,mutate:VC.MAJOR,description:"Validate the JET Version compatibility range"},packContent:{callback:packContent,add:VC.MINOR,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.SEMVER,description:"Validate the member components of a JET Pack"},packBundle:{callback:packBundle,add:VC.MINOR,remove:VC.MAJOR,mutate:VC.NEVER,description:"Validate the list of bundles defined by a JET Pack"},packBundleContent:{callback:packBundleContent,add:VC.MAJOR,remove:VC.MAJOR,mutate:VC.NEVER,description:"Validate the entries of each bundle defined by a JET Pack"},dependencies:{callback:dependencies,add:VC.MAJOR,remove:VC.MINOR,mutate:VC.SEMVER,description:"Validate the dependencies of a component"},properties:{callback:properties,add:VC.MINOR,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the properties of a component"},propertyDefault:{callback:propertyDefault,add:VC.PATCH,remove:VC.MAJOR_DEPRECATED,mutate:VC.MAJOR_DEPRECATED,description:"Validate the defaults assigned to properties of a component"},propertyRequired:{callback:propertyRequired,add:VC.MAJOR,remove:VC.PATCH,mutate:VC.NEVER,description:"Validate the required states of a components properties"},propertyReadOnly:{callback:propertyReadOnly,add:VC.MAJOR,remove:VC.PATCH,mutate:VC.NEVER,description:"Validate the read-only states of a components properties"},propertyWriteback:{callback:propertyWriteback,add:VC.PATCH,remove:VC.NEVER,mutate:VC.NEVER,description:"Validate the writeback states of a components properties"},propertyType:{callback:propertyType,add:VC.PATCH,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the assigned types of a components properties"},propertyEnumeration:{callback:propertyEnumeration,add:VC.PATCH,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.MAJOR,description:"Validate the enumValues of a components properties"},methods:{callback:methods,add:VC.MINOR,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the list of methods exposed by the component"},methodParams:{callback:methodParams,add:VC.MINOR,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the list of parameters passed to methods exposed by the component"},methodParamTypes:{callback:methodParamTypes,add:VC.PATCH,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the list of parameters passed to methods exposed by the component"},methodReturnType:{callback:methodReturnType,add:VC.PATCH,remove:VC.MAJOR,mutate:VC.NEVER,description:"Validate the return type of the methods exposed by the component"},events:{callback:events,add:VC.MINOR,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the list of events exposed by the component"},eventPayload:{callback:eventPayload,add:VC.MINOR,remove:VC.MAJOR,mutate:VC.NEVER,description:"Validate the event detail of events exposed by the component"},eventBubbles:{callback:eventBubbles,add:VC.MINOR,remove:VC.NEVER,mutate:VC.NEVER,description:"Validate the event bubbles flag"},eventCancelable:{callback:eventCancelable,add:VC.MINOR,remove:VC.NEVER,mutate:VC.NEVER,description:"Validate the event cancelable flag"},slots:{callback:slots,add:VC.MINOR,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the list of slots exposed by the component"},slotDataVariable:{callback:slotDataVariable,add:VC.PATCH,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the list of attributes in a slot data attribute exposed by the component"},slotDataVariableType:{callback:slotDataVariableType,add:VC.NEVER,remove:VC.NEVER,mutate:VC.NEVER,description:"Validate the type assigned to each attributes in a slot data attribute exposed by the component"},dynamicSlots:{callback:dynamicSlots,add:VC.MINOR,remove:VC.MAJOR,mutate:VC.NEVER,description:"Validate the presence of the dynamicSlots marker on the component"},styleVariables:{callback:styleVariables,add:VC.MINOR,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the list of styleVariables supported by the component"},styleClasses:{callback:styleClasses,add:VC.MINOR,remove:VC.MAJOR_2_DEPRECATED,mutate:VC.NEVER,description:"Validate the list of styleClasses supported by the component"},publicModules:{callback:publicModules,add:VC.MINOR,remove:VC.MAJOR,mutate:VC.NEVER,description:"Validate the list of publicModules exposed by a resource component"},componentType:{callback:componentType,add:VC.NEVER,remove:VC.NEVER,mutate:VC.NEVER,description:"Validate the component type has not been changed"}};const ruleDef=function(){};ruleDef.prototype.getName=function(){return"jetwco-component-api-controlled-changes"};ruleDef.prototype.getShortDescription=function(){return"Check components for API changes based on a configurable rule-set"};ruleDef.prototype.getDescription=function(){return"This audit compares the APIs exposed by components with older versions in the Exchange Catalog according to a set of rules which determine when given changes, such as the removal of deprecated properties can be made"};ruleDef.prototype.register=function(context){return{startaudit:this._auditPrep,closedownRP:this._dealWithCache,compjson:this._doAudit,Program:this._doVComponentAudit}};ruleDef.prototype._auditPrep=function(context){this.exchangeURL=CommonImpl.checkForExchangeConfiguration(context);if(this.exchangeURL){this.acCache=new AuditComponentCacheManager(context,this.getName(),this.exchangeURL);this.componentExceptions=context.rulePack.getRuleCustomOption("exceptions");const ipr=context.rulePack.getRuleCustomOption("includePreReleases");if(ipr&&ipr===true){this.productionOnly=false}else{this.productionOnly=true}const ruleOverrides=context.rulePack.getRuleCustomOption("controls");this.controlRules={};if(Object.keys(ruleOverrides).length>0){for(const[controlRule,controlRuleDef]of Object.entries(CONTROL_PROFILE)){this.controlRules[controlRule]={...controlRuleDef};if(ruleOverrides.hasOwnProperty(controlRule)){for(const[verb,override]of Object.entries(ruleOverrides[controlRule])){this.controlRules[controlRule][verb]=override}}}}else{this.controlRules={...CONTROL_PROFILE}}const activeControls=context.rulePack.getRuleCustomOption("applyConditions");if(activeControls&&Array.isArray(activeControls)){Object.keys(this.controlRules).forEach(controlRule=>{if(!activeControls.includes(controlRule)){delete this.controlRules[controlRule]}})}this.requiredCheckVersions=CaccHelpers.calcRequiredVersionMatrix(this.controlRules)}else{const issue=new context.Issue(`Invalid or Missing exchangeURL value specified for the rule ${context.ruleName}, rule execution is disabled`);issue.setMsgKey(`${context.ruleName}_2`);issue.setSeverity("critical");context.reporter.addIssue(issue,context)}};ruleDef.prototype._dealWithCache=function(context){const shouldClearCache=context.rulePack.getRuleCustomOption("clearCache",this.getName());if(shouldClearCache&&this.acCache){this.acCache.removeLocalCache()}};ruleDef.prototype._doAudit=function(context){if(context.suppData.obj){const meta=context.suppData.obj;const fullName=Utils.getComponentFullName(meta);const componentType=Utils.getComponentType(meta);this._conductChecks(context,fullName,componentType,meta)}};ruleDef.prototype._doVComponentAudit=function(context){const componentInfo=TsxUtils.matchToVComponentImpl(context,context.filepath);if(componentInfo){const meta={...componentInfo};if(componentInfo.dependencies){meta.dependencies=componentInfo.dependencies.reduce((asMap,entry)=>{asMap[entry.name]=entry.range;return asMap},{})}this._conductChecks(context,componentInfo.fullName,componentInfo.type,meta)}};ruleDef.prototype._conductChecks=function(context,fullName,componentType,componentMeta){const{msgLib,semVerUtils}=context.utils;const privateComponent=componentMeta.subcomponentType&&componentMeta.subcomponentType==="packPrivate";const skipPrivate=context.rulePack.getRuleCustomOption("ignorePackPrivate",context.ruleName);const overrideTypeMappings=applyCustomMappings(context.rulePack.getRuleCustomOption("typeMappings",context.ruleName));if(skipPrivate&&privateComponent){msgLib.info(`packPrivate component ${fullName} skipped due to ignorePackPrivate flag in  ${context.ruleName} configuration`)}else{const baselines=this._grabRequiredVersions(context,fullName,componentMeta.version,this.requiredCheckVersions,this.productionOnly);if(baselines&&baselines.latest.version){const compareMetadata=CaccHelpers.deriveConstraintMetadata(baselines);const componentVersionDiff=CaccHelpers.semanticDiff(semVerUtils,componentMeta.version,compareMetadata.version);try{for(const[controlRuleName,controlRuleDef]of Object.entries(this.controlRules)){if(!this._isExemptFromCheck(fullName,controlRuleName)){msgLib.info(`${context.ruleName}(${controlRuleName}): ${controlRuleDef.description}`);const errorMsgContext=CaccHelpers.getStdErrorContext(fullName,controlRuleName);this._executeCheck(controlRuleName,controlRuleDef,semVerUtils,this.acCache,componentMeta,compareMetadata,componentVersionDiff,baselines,componentType,fullName,errorMsgContext,overrideTypeMappings)}}}catch(auditDetail){const issue=new context.Issue(`API change for ${fullName}@${componentMeta.version}: ${auditDetail}`);issue.setMsgKey(`${context.ruleName}_1`);context.reporter.addIssue(issue,context)}}}};ruleDef.prototype._grabRequiredVersions=function(context,fullName,versionUnderTest,requiredVersions,productionVersionsOnly){const{msgLib,semVerUtils}=context.utils;const checkVersions={"never":{isNeeded:true,version:null,metadata:null},"latest":{isNeeded:true,version:null,metadata:null},"patch":{isNeeded:requiredVersions.patch,version:null,metadata:null},"minor":{isNeeded:requiredVersions.minor,version:null,metadata:null},"major":{isNeeded:requiredVersions.major,version:null,metadata:null},"major+":{isNeeded:requiredVersions.major,version:null,metadata:null},"major2+":{isNeeded:requiredVersions.major2,version:null,metadata:null},"major3+":{isNeeded:requiredVersions.major3,version:null,metadata:null}};try{const versionsToCache=new Set;const netUtils=new NetUtils;const existingVersionsResp=netUtils.loadUrl(`${this.exchangeURL}components/${fullName}/versions`);if(existingVersionsResp.statusCode===200&&existingVersionsResp.body){const existingVersions=JSON.parse(existingVersionsResp.body);if(existingVersions.items&&existingVersions.items.length>0){const{major:vutMajor,minor:vutMinor,patch:vutPatch,prerelease:vutPreRel,version:vutVersion}=semVerUtils.parse(versionUnderTest);let historicalVersions;if(productionVersionsOnly){historicalVersions=existingVersions.items.reverse().filter(versionString=>{const{prerelease}=semVerUtils.parse(versionString);return prerelease.length===0})}else{historicalVersions=existingVersions.items.reverse()}const majorSequence=this._getOffsetMajorVersions(semVerUtils,historicalVersions,vutMajor);for(let i=0;i<historicalVersions.length;i++){const{major:hvMajor,minor:hvMinor,patch:hvPatch,prerelease:hvPreRel,version:hvVersion}=semVerUtils.parse(historicalVersions[i]);if(semVerUtils.lt(hvVersion,vutVersion)){if(!checkVersions.latest.version){checkVersions.latest.version=hvVersion;versionsToCache.add(hvVersion)}if(checkVersions.patch.isNeeded&&!checkVersions.patch.version){checkVersions.patch.version=hvVersion;versionsToCache.add(hvVersion)}if(checkVersions.minor.isNeeded&&!checkVersions.minor.version){if(hvMinor<vutMinor||hvMajor<vutMajor){checkVersions.minor.version=hvVersion;versionsToCache.add(hvVersion)}}if(checkVersions.major.isNeeded&&!checkVersions.major.version){if(majorSequence[1]&&hvMajor===majorSequence[1]){checkVersions.major.version=hvVersion;checkVersions["major+"].version=hvVersion;versionsToCache.add(hvVersion)}}if(checkVersions["major2+"].isNeeded&&!checkVersions["major2+"].version){if(majorSequence[2]&&hvMajor===majorSequence[2]){checkVersions["major2+"].version=hvVersion;versionsToCache.add(hvVersion)}}if(checkVersions["major3+"].isNeeded&&!checkVersions["major3+"].version){if(majorSequence[3]&&hvMajor===majorSequence[3]){checkVersions["major3+"].version=hvVersion;versionsToCache.add(hvVersion);break}}}}versionsToCache.forEach(requiredVersion=>{const cachedDef=this.acCache.getComponentMetadata(fullName,requiredVersion);if(cachedDef){for(const[controlValue,settings]of Object.entries(checkVersions)){if(settings.isNeeded&&settings.version===requiredVersion){checkVersions[controlValue].metadata=cachedDef}}}})}}else if(existingVersionsResp.statusCode===404||existingVersionsResp.eCode==="ENOTFOUND"){msgLib.info(`The component ${fullName} does not exist in the Exchange and so cannot be API verified against older versions. It will be skipped`);if(!this.componentExceptions){this.componentExceptions={}}this.componentExceptions[fullName]=["*"];return}}catch(e){msgLib.error(`Unexpected error processing rule ${context.ruleName} for component ${fullName}:\n${JSON.stringify(e)}`)};return checkVersions};ruleDef.prototype._getOffsetMajorVersions=function(semVerUtils,versions,baseMajorVersion){const majors=[baseMajorVersion];let lastMajor=baseMajorVersion;versions.forEach(stringVersion=>{const{major:parsedMajor}=semVerUtils.parse(stringVersion);if(parsedMajor<lastMajor){lastMajor=parsedMajor;majors.push(parsedMajor)}});return majors};ruleDef.prototype._executeCheck=function(controlRuleName,controlRuleDef,semver,cacheManager,metadata,compareMetadata,componentVersionDiff,baselines,componentType,componentName,errorMsgContext,overrideTypeMappings){const result=controlRuleDef.callback.call(this,controlRuleName,controlRuleDef,semver,cacheManager,metadata,compareMetadata,componentVersionDiff,baselines,componentType,componentName,errorMsgContext,overrideTypeMappings);if(result){throw result}};ruleDef.prototype._isExemptFromCheck=function(fullName,controlRuleName){let exempt=false;let nameMatch;if(this.componentExceptions){if(this.componentExceptions.hasOwnProperty(fullName)){nameMatch=fullName}else{const wildcardNames=Object.keys(this.componentExceptions).reduce((wildcardList,wc)=>{if(wc.endsWith("*")){wildcardList.push(wc.substring(0,wc.length-1))}return wildcardList},[]);if(wildcardNames.length>0){wildcardNames.forEach(wildcard=>{if(fullName.startsWith(wildcard)){nameMatch=wildcard+"*"}})}}if(nameMatch){if(this.componentExceptions[nameMatch].includes("*")||this.componentExceptions[nameMatch].includes(controlRuleName)){exempt=true}}}return exempt};module.exports=ruleDef;