/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const RULE_NAME="spoc-dom-manip";const PROP_NAMES=["innerHTML","outerHTML"];const FUNCS_DOM_MANIP=["html","prepend","prependTo","before","after","insertAfter","append","appendTo","wrap","wrapAll","wrapInner","replaceAll","replaceWith","write","writeln","attr","setAttribute"];const FUNCS_OF_INTEREST=["eval","Function","setTimeout","setInterval","random","createElement","write","writeln"];const FUNCS_SPECIAL_CASE=["appendChild"];const FUNCS_AJAX=["XMLHttpRequest","ActiveXObject"];const SAFE_ATTR_NAMES=["align","alink","alt","bgcolor","border","cellpadding","cellspacing","class","color","cols","colspan","coords","dir","face","height","hspace","id","ismap","lang","marginheight","marginwidth","multiple","nohref","noresize","noshade","nowrap","ref","rel","rev","role","rows","rowspan","scrolling","shape","span","summary","tabindex","title","usemap","valign","value","vlink","vspace","width"];const UNSAFE_ATTR_NAMES=["action","allow","async","background","charset","cite","data","formaction","icon","ping","poster","src","srcdoc","srcset","style"];const FUNC_EXPR="FunctionExpression";const ARROW_FUNC_EXPR="ArrowFunctionExpression";const IDENTIFIER="Identifier";const LITERAL="Literal";const BLOCK_STMT="BlockStatement";const CALL_EXPR="CallExpression";const MEMBER_EXPR="MemberExpression";const TAG_OK="@htmlupdateok";const TAG_REVIEW="@htmlupdatereview";const TAG_XSS_F_POS="@xssfalsepositive";const TAG_RANDOM="@randomnumberok";function Rule(){};Rule.prototype.getName=function(){return RULE_NAME};Rule.prototype.getShortDescription=function(){return"Reports DOM manipulations in JS"};Rule.prototype.getDescription=function(){return"Reports on DOM manipulations and optionally provides the ability to tag with @HTMLUpdateOK."};Rule.prototype.register=function(regCtx){if(!this._checkRuleOverrides(regCtx)){return false}return{AssignmentExpression:this._handleDom,CallExpression:this._handleCall,NewExpression:this._handleNewExpr,MemberExpression:this._handleMemExpr,VariableDeclaration:this._handleVarDecl,VariableDeclarator:this._handleVarDecl}};Rule.prototype._handleDom=function(context,node){if(node.left){this._handleDom2(context,node.left,node.right,true)}};Rule.prototype._handleMemExpr=function(context,node){var obj,funcName;if(node.parent&&node.parent.type==="ConditionalExpression"){obj=node.parent;if(obj.consequent&&obj.consequent.property&&obj.consequent.property.type===IDENTIFIER){obj=obj.consequent.property;if(isDomManip(obj.name)){if(this._settings.new.showGetters){this._checkAndEmit(context,obj,{msg:"DOM manipulation getter '"+obj.name+"'"})}}else if(isDomManipFunc(obj.name)){funcName:obj.property&&obj.property.type===IDENTIFIER?obj.property.name:obj.name;if(funcName){this._checkAndEmit(context,obj,{isFunc:true,funcName:funcName,msg:"DOM manipulation function "+funcName+"()"})}}else if(isFuncOfInterest(obj.name)){this._checkAndEmit(context,obj,{funcName:obj.property&&obj.property.type===IDENTIFIER?obj.property.name:obj.name,isFunc:true,funcName:funcName,msg:"DOM manipulation function "+funcName+"()"})}}if(obj.alternate&&obj.alternate.property&&obj.alternate.property.type===IDENTIFIER){obj=obj.alternate.property;if(isDomManip(obj.name)){if(this._settings.new.showGetters){this._checkAndEmit(context,obj,{msg:"DOM manipulation getter '"+obj.name+"'"})}}else if(isDomManipFunc(obj.name)){funcName=obj.property&&obj.property.type===IDENTIFIER?obj.property.name:obj.name;this._checkAndEmit(context,obj,{isFunc:true,funcName:funcName,msg:"DOM manipulation function "+funcName+"()"})}}}else{this._checkCookie(context,node)}};Rule.prototype._handleCall=function(context,node){var obj,funcOfInterest,funcName;if(node.callee){obj=node.callee.property?node.callee.property:node.callee;funcName=obj.name;funcOfInterest=isFuncOfInterest(funcName);if(isDomManipFunc(funcName)||funcOfInterest){if(this._checkHtmlFunc(context,node)||this._checkDocumentWrite(context,node)||this._checkCreateElement(context,node)||this._checkTimeFuncs(context,node)||this._checkMathRandom(context,node)||this._checkEval(context,node)||this._checkSafeSetAttrName(context,node)){return}this._checkAndEmit(context,obj,{isFunc:true,funcName:funcName,msg:(!funcOfInterest?"DOM manipulation ":"")+"function "+funcName+"()"})}}};Rule.prototype._handleNewExpr=function(context,node){var callee,funcName;if(node.callee){callee=node.callee;if(isFuncAjax(callee.name)){funcName=_getFuncName(callee);this._checkAndEmit(context,callee,{isFunc:true,funcName:funcName,msg:"function "+funcName+"()"})}else{if(callee.type===IDENTIFIER&&callee.name==="Function"){funcName=_getFuncName(callee);this._checkAndEmit(context,callee,{isFunc:true,funcName:funcName,msg:"function "+funcName+"()"})}}}};Rule.prototype._handleDom2=function(context,nodeLeft,nodeRight,isSetter){var node;if(nodeLeft.property){node=nodeLeft.property;if(node.type===IDENTIFIER){if(isDomManip(node.name)){if(!this._checkAssignedValue(context,nodeRight)){if(isSetter||this._settings.new.showGetters){this._checkAndEmit(context,node,{msg:"DOM manipulation "+(isSetter?"setter":"getter")+" '"+node.name+"'"})}}}}}else if(nodeRight.type===LITERAL&&nodeRight.value==="script"){this._checkAndEmit(context,nodeRight,{msg:"var assignment of 'script'"})}};Rule.prototype._handleVarDecl=function(context,node){var i,decl;if(node.declarations){for(i=0;i<node.declarations.length;i++){decl=node.declarations[i];if(!decl.init){continue}if(decl.init.type===LITERAL&&decl.init.value==="script"){this._checkAndEmit(context,decl,{isFunc:false,msg:"var '"+decl.id.name+"' initialized with 'script'"})}}}};Rule.prototype._check=function(context,obj,isFunc,setter,isDomManipFunc,isVar,isScriptAssign,isCookie){var line,tagType,tagged,isTagged,funcName,issue,rowcol;var utils=context.utils.utils;line=isolateLine(context.data,obj.end);tagType=getLineTag(line);tagged=tagType===TAG_OK?"UpdateOK tagged":tagType===TAG_XSS_F_POS?"XSS tagged":tagType===TAG_RANDOM?"RandomOK tagged":"untagged";isTagged=tagged!=="untagged";if(tagType===TAG_REVIEW){issue=new context.Issue("SPOC - obsolete tag '@HTMLUpdateReview' - remove");issue.setSeverity("info");rowcol=utils.getRowColFromIndex(context.data,obj.start);issue.setPosition(rowcol.row,rowcol.col,obj.start,obj.end);context.reporter.addIssue(issue,context);issue=null;tagType=""}if(isFunc){funcName=obj.property&&obj.property.type===IDENTIFIER?obj.property.name:obj.name;if(funcName==="random"&&tagType&&tagType!==TAG_RANDOM){issue=new context.Issue("SPOC - Math.random() is tagged incorrectly (use '@RandomNumberOK')");context.reporter.addIssue(issue,context);return}}if(!tagType&&this._settings.new.show||tagType===TAG_OK&&this._settings.existing.showUpdate||tagType===TAG_XSS_F_POS&&this._settings.existing.showXSS||tagType===TAG_RANDOM&&this._settings.existing.showRandom){issue=null;if(isFunc){issue=new context.Issue("SPOC - "+tagged+(isDomManipFunc?"DOM manipulation":"")+" function "+funcName+"()")}else if(isVar){issue=new context.Issue("SPOC - "+tagged+" var '"+obj.id.name+"' initialized with 'script'")}else if(isScriptAssign){issue=new context.Issue("SPOC - "+tagged+" var assignment of 'script'")}else if(isCookie){issue=new context.Issue("SPOC - "+tagged+" use of 'document.cookie'")}else if(setter||this._settings.new.showGetters){issue=new context.Issue("SPOC - "+tagged+" DOM manipulation "+(setter?"setter":"getter")+" '"+obj.name+"'")}if(issue){rowcol=utils.getRowColFromIndex(context.data,obj.property?obj.property.start:obj.start);issue.setPosition(rowcol.row,rowcol.col,obj.start,obj.end);if(isTagged){issue.setSeverity("info")}context.reporter.addIssue(issue,context)}}};Rule.prototype._checkAndEmit=function(context,obj,info){var line,tagType,tagged,isTagged,issue,rowcol;var utils=context.utils.utils;line=isolateLine(context.data,obj.end);tagType=getLineTag(line);tagged=tagType===TAG_OK?"UpdateOK tagged":tagType===TAG_XSS_F_POS?"XSS tagged":tagType===TAG_RANDOM?"RandomOK tagged":"untagged";isTagged=tagged!=="untagged";if(tagType===TAG_REVIEW){issue=new context.Issue("SPOC - obsolete tag '@HTMLUpdateReview' - remove");issue.setSeverity("info");rowcol=utils.getRowColFromIndex(context.data,obj.start);issue.setPosition(rowcol.row,rowcol.col,obj.start,obj.end);context.reporter.addIssue(issue,context);issue=null;tagType=""}issue=null;if(info.funcName){}if(!tagType&&this._settings.new.show||tagType===TAG_OK&&this._settings.existing.showUpdate||tagType===TAG_XSS_F_POS&&this._settings.existing.showXSS||tagType===TAG_RANDOM&&this._settings.existing.showRandom){issue=new context.Issue("SPOC - "+tagged+" "+info.msg);if(issue){rowcol=utils.getRowColFromIndex(context.data,obj.property?obj.property.start:obj.start);issue.setPosition(rowcol.row,rowcol.col,obj.start,obj.end);if(isTagged){issue.setSeverity("info")}context.reporter.addIssue(issue,context)}}};Rule.prototype._checkFuncOKUntagged=function(context,obj){var line,tagType,issue,rowcol;if(!this._settings.existing.showOkUntagged){return}line=isolateLine(context.data,obj.end);tagType=getLineTag(line);if(!tagType&&this._settings.new.show||tagType!==TAG_OK&&this._settings.existing.showUpdate||tagType!==TAG_XSS_F_POS&&this._settings.existing.showXSS){issue=new context.Issue("SPOC - "+" DOM manipulation function "+obj.name+"() is OK, but untagged");if(issue){rowcol=context.utils.utils.getRowColFromIndex(context.data,obj.start);issue.setPosition(rowcol.row,rowcol.col,obj.start,obj.end);issue.setSeverity("info");context.reporter.addIssue(issue,context)}}};Rule.prototype._checkAssignedValue=function(context,node){var v,ret=false;if(node.type===LITERAL){v=node.value;if(context.utils.utils.getType(v)==="string"){if(v===""){ret=true}}}return ret};Rule.prototype._checkHtmlFunc=function(context,node){var callee,ret=false;callee=node.callee;if(callee.property&&callee.property.type===IDENTIFIER){if(callee.property.name==="html"){if(node.arguments&&node.arguments.length){this._checkAndEmit(context,callee.property,{isFunc:true,funcName:"html",msg:"DOM manipulation function html() used as a setter"})}ret=true}}return ret};Rule.prototype._checkDocumentWrite=function(context,node){var callee,meth,ret=false;callee=node.callee;if(callee.property&&callee.property.type===IDENTIFIER){meth=callee.property.name;if(meth==="write"||meth==="writeln"){if(callee.type===MEMBER_EXPR&&callee.object.type===IDENTIFIER&&callee.object.name==="document"){this._checkAndEmit(context,callee.property,{isFunc:true,funcName:meth,msg:"DOM manipulation function document."+meth+"()"});ret=true}}}return ret};Rule.prototype._checkCreateElement=function(context,node){var callee,meth,args,arg,ret=false;callee=node.callee;if(callee.property&&callee.property.type===IDENTIFIER){meth=callee.property.name;if(meth==="createElement"){if(true){args=node.arguments;if(args.length>=1){arg=args[0];if(arg.type===LITERAL){if(arg.value!=="script"){return true}else{this._checkAndEmit(context,callee.property,{isFunc:true,funcName:"createElement",msg:"function createElement() used to create a 'script' element"})}return true}else if(arg.type==="ConditionalExpression"){if(arg.consequent.type===LITERAL&&arg.consequent.value!=="script"&&arg.alternate.type===LITERAL&&arg.alternate.value!=="script"){return true}}}this._checkAndEmit(context,callee.property,{isFunc:true,funcName:"createElement",msg:"function createElement() may potentially be used to create a 'script' element"});ret=true}}}return ret};Rule.prototype._checkCookie=function(context,node){var obj,prop,ret=false;obj=node.object;if(obj&&obj.type===IDENTIFIER&&obj.name==="document"){prop=node.property;if(prop&&prop.type===IDENTIFIER&&prop.name==="cookie"){this._checkAndEmit(context,prop,{msg:"use of 'document.cookie'"});ret=true}}return ret};Rule.prototype._checkAppendChild=function(context,node){var callee,meth,obj,ret=false;callee=node.callee;if(callee.property&&callee.property.type===IDENTIFIER){meth=callee.property.name;if(meth==="appendChild"){if(callee.type===MEMBER_EXPR&&callee.object.type===IDENTIFIER&&callee.object.name==="document"){obj=callee.object;if(obj.property&&obj.property.type===IDENTIFIER&&obj.name==="head"){this._checkAndEmit(context,callee.property,{isFunc:true,funcName:"appendchild",msg:"DOM manipulation function appendChild()"});ret=true}}else{this._checkAndEmit(context,callee.property,{isFunc:true,funcName:"appendChild",msg:"DOM manipulation function appendChild()"});ret=true}}}return ret};Rule.prototype._checkTimeFuncs=function(context,node){var obj,callee,meth,args,arg,ret=false;callee=node.callee;obj=callee.property?callee.property:callee;if(obj.type===IDENTIFIER){meth=obj.name;if(meth!=="setTimeout"&&meth!=="setInterval"){return false}if(node.arguments){args=node.arguments;if(args.length>=1){arg=args[0];if(arg.type!==FUNC_EXPR&&arg.type!==ARROW_FUNC_EXPR){if(arg.type===LITERAL||arg.type===CALL_EXPR||!_isIdentAFunc(node,arg.name)){this._checkAndEmit(context,callee,{isFunc:false,funcName:meth,msg:"function "+meth+"()"})}}else{}ret=true}}}return ret};Rule.prototype._checkMathRandom=function(context,node){var obj,callee,meth,ret=false;callee=node.callee;obj=callee.property?callee.property:callee;if(callee.object&&callee.object.type===IDENTIFIER&&callee.object.name==="Math"){meth=obj.name;if(meth==="random"){this._checkAndEmit(context,obj,{isFunc:true,funcName:"Math.random",msg:"function Math.random()"});ret=true}}return ret};Rule.prototype._checkEval=function(context,node){var obj,callee,meth,ret=false;callee=node.callee;obj=callee.property?callee.property:callee;meth=obj.name;if(meth==="eval"){this._checkAndEmit(context,obj,{isfunc:true,funcName:"eval",msg:"function eval()"});ret=true}return ret};Rule.prototype._checkAjax=function(context,node){var obj,callee,meth,ret=false;callee=node.callee;obj=callee.property?callee.property:callee;meth=obj.name;if(meth==="XMLHttpRequest"||meth==="ActiveXObject"){this._checkAndEmit(context,obj,{isFunc:true,funcName:meth,msg:"function "+meth+"()"});ret=true}return ret};Rule.prototype._checkSafeSetAttrName=function(context,node){var callee,meth,args,arg,msg,ret=false;callee=node.callee;if(callee.property&&callee.property.type===IDENTIFIER){meth=callee.property.name;if(meth==="setAttribute"||meth==="attr"){args=node.arguments;if(args.length<2){return true}arg=args[0];if(arg.type===LITERAL){if(isSafeSetAttrName(arg.value,meth==="setAttribute")){return true}if(isUnsafeSetAttrName(arg.value,args[1].value)){if(arg.value.startsWith("on")){if(context.utils.utils.getType(args[1])==="function"){this._checkFuncOKUntagged(context,callee.property)}else{this._checkAndEmit(context,callee.property,{isFunc:true,funcName:meth,msg:"function "+meth+"('"+arg.value+"', ...)"})}}else{this._checkAndEmit(context,callee.property,{isFunc:true,funcName:meth,msg:"function "+meth+"('"+arg.value+"', ...)"})}}ret=true}if(!ret){if(args&&args.length){if(args[0].type===LITERAL){msg=args[0].raw+", ..."}else{msg="expr, ... "}}this._checkAndEmit(context,callee.property,{isFunc:true,funcName:meth,msg:"function "+meth+"("+msg+")"});ret=true}}}return ret};Rule.prototype._checkRuleOverrides=function(regCtx){var customOpts;var utils=regCtx.utils.utils;function _emitMsg(s){regCtx.utils.msgLib.error(`Rule ${RULE_NAME} registration - rule option '${s}' - expected an object`)}if(customOpts=regCtx.ruleOpts.customOpts){this._settings=customOpts.settings}if(!this._settings||!utils.isObject(this._settings)){_emitMsg("settings");return false}let opt=this._settings["new"];if(!opt||!utils.isObject(opt)){_emitMsg("settings.new");return false}opt=this._settings.existing;if(!opt||!utils.isObject(opt)){_emitMsg("settings.existing");return false}return true};function isDomManip(name){return PROP_NAMES.includes(name)};function isDomManipFunc(name){return FUNCS_DOM_MANIP.includes(name)};function isFuncOfInterest(name){return FUNCS_OF_INTEREST.includes(name)};function isFuncAjax(name){return FUNCS_AJAX.includes(name)};function isSpecialCaseFunc(name){return FUNCS_SPECIAL_CASE.includes(name)};function isSafeSetAttrName(name,isSetAttribute){return name.startsWith("data-")||name.startsWith("aria-")||SAFE_ATTR_NAMES.includes(name)||name==="for"||name.startsWith("oj-")};function isUnsafeSetAttrName(attr,value){value=value||"";if(typeof value==="string"){value=value.trim()}return attr.startsWith("on")||UNSAFE_ATTR_NAMES.includes(attr)||attr==="href"&&value!=="#"&&value!==""};function _getFuncName(obj){return obj.property&&obj.property.type===IDENTIFIER?obj.property.name:obj.name};function _isIdentAFunc(node,funcName){var n,decls,decl,i,j,isFunc=false;node=node.parent;while(node){if(!node){break}if(node.type!==BLOCK_STMT){if(node.type!=="Program"){node=node.parent;continue}for(i=0;i<node.body.length;i++){n=node.body[i];if(n.type==="FunctionDeclaration"){isFunc=n.id&&n.id.type===IDENTIFIER&&n.id.name===funcName;if(isFunc){return true}}}return false}if(node.body){for(i=0;i<node.body.length;i++){n=node.body[i];if(n.type==="FunctionDeclaration"){isFunc=n.id&&n.id.type===IDENTIFIER&&n.id.name===funcName;if(isFunc){return true}}}for(i=0;i<node.body.length;i++){n=node.body[i];if(n.type==="VariableDeclaration"){decls=n.declarations;if(decls){for(j=0;j<decls.length;j++){decl=decls[j];if(decl.type==="VariableDeclarator"){if(decl.id.type===IDENTIFIER&&decl.id.name===funcName){return decl.init&&(decl.init.type===FUNC_EXPR||decl.init.type===ARROW_FUNC_EXPR)}}}}isFunc=n.id&&n.id.type===IDENTIFIER&&n.id.name===funcName;if(isFunc){return true}}}}node=node.parent}return false};function isolateLine(data,endIndex){var i,c,len=data.length;for(i=endIndex;i<len;i++){c=data.charAt(i);if(c==="\n"||c==="\r"){break}}return data.substring(endIndex,i)};function getLineTag(line){var lc=line.toLowerCase();return lc.includes(TAG_OK)?TAG_OK:lc.includes(TAG_REVIEW)?TAG_REVIEW:lc.includes(TAG_XSS_F_POS)?TAG_XSS_F_POS:lc.includes(TAG_RANDOM)?TAG_RANDOM:""};module.exports=Rule;