/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const{LENGTH_PROP_SET}=require("./helpers/cssHelper");const RULENAME="oj-css-style-abs-units";const DESCRIPTION="In general it is preferable to use relative length units in place of absolute units. "+"Specifically, 'rem' units are recommended because they proportionately adjust to the root "+"font size.  'rem' can be used for anything that takes a length unit - e.g. font-size, line-height, "+"width, height, padding, margin, etc.  However, there are cases where absolute units are "+"required.  To ignore certain selectors, use the 'ignore'/'accept' sub-properties of the "+"configuration property 'ruleMods'.<br><br>"+"e.g. <em>\"ruleMods\" : { &nbsp;\"ignore\" : &nbsp;[\"-image$\"] &nbsp;}</em><br><br>"+"this will ignore all selectors ending with <em>\"-image$\"</em>";const SHORT_DESCRIPTION="In general it is preferable to use relative length units in place of absolute units.";const ABS_UNITS=["px","cm","mm","in","pt","pc","all"];const SELECTORLIST="SelectorList";const SELECTOR="Selector";const CLASS_SELECTOR="ClassSelector";const ID_SELECTOR="IdSelector";const FUNCTION="Function";const DIMENSION="Dimension";class Rule{getName(){return RULENAME}getDescription(){return DESCRIPTION}getShortDescription(){return SHORT_DESCRIPTION}register(regCtx){if(!this._checkRuleOptions(regCtx)){return false}this._CssUtils=regCtx.utils.CssUtils;return{"css-rule":this._onRule}}_onRule(ruleCtx,rule){var block,decl,i;block=rule.block.children;for(i=0;i<block.length;i++){decl=block[i];if(decl.type==="Declaration"&&LENGTH_PROP_SET.has(decl.property)){if(!_isCssVar(decl)){this._prop(ruleCtx,rule,decl)}}}}_prop(ruleCtx,rule,decl){if(decl.value&&decl.value.type==="Value"&&decl.value.children){this._children(ruleCtx,rule,decl.property,decl.value.children)}}_children(ruleCtx,rule,prop,childs){var ch,i;for(i=0;i<childs.length;i++){ch=childs[i];if(ch.type===DIMENSION&&ABS_UNITS.includes(ch.unit)&&this._okPropFail(rule,ch)){this._emitIssue(ruleCtx,prop,ch)}else if(ch.type===FUNCTION&&!_isColorFunc(ch.name)){this._func(ruleCtx,rule,prop,ch)}}}_func(ruleCtx,rule,prop,fnNode){var a,decl,i;a=fnNode.children;for(i=0;i<a.length;i++){decl=a[i];if(decl.type===DIMENSION&&ABS_UNITS.includes(decl.unit)&&this._okPropFail(rule,decl)){this._emitIssue(ruleCtx,prop,decl)}}}_emitIssue(ruleCtx,prop,dim){var val,issue;val=parseInt(dim.value);if(val){issue=new ruleCtx.Issue(`CSS value '${val}${dim.unit}' (${prop}) :  - best practices: replace absolute unit '${dim.unit}' with a relative unit (e.g. 'rem' or '%')`)}else{issue=new ruleCtx.Issue(`CSS value '0${dim.unit}' (${prop}) :  - best practices: remove '${dim.unit}' for zero quantity`);issue.setMsgKey(RULENAME+"_1");issue.setSeverity("minor")}issue.setPosition(this._CssUtils.getPosition(ruleCtx,dim.loc));ruleCtx.reporter.addIssue(issue,ruleCtx)}_okPropFail(rule,decl){var unit,val,sels,list,b,i,j;val=typeof decl.value==="string"?parseInt(decl.value):val;if(this._unitOpts){unit=this._unitOpts[decl.unit];if(unit===undefined){unit=this._unitOpts["all"]}b=typeof unit==="number"?val===0?true:Math.abs(val)>unit:false}else{b=true}if(b&&(this._ignore||this._accept)){sels=_getSelectors(rule);if(sels){list=this._ignore||this._accept;for(i=0;i<list.length;i++){for(j=0;j<sels.length;j++){if(list[i].test(sels[j])){return this._accept?b:false}}}}return!this._accept}return b}_checkRuleOptions(regCtx){var customOpts,units,sels,accept,ign,unit,th,b;if(!(customOpts=regCtx.ruleOpts.customOpts)){return true}if(units=customOpts["absunits"]){if(!regCtx.utils.utils.isObject(units)){regCtx.utils.msgLib.error(`Rule ${regCtx.ruleName} :  option 'absunits' is not an object`);this._fail=true;return false}for(unit in units){if(!ABS_UNITS.includes(unit)){regCtx.utils.msgLib.error(`Rule ${regCtx.ruleName} :  option 'absunits' value '${unit}' is not absolute`);b=true;continue}if(units.hasOwnProperty(unit)){th=units[unit];if(typeof th!=="number"){regCtx.utils.msgLib.error(`Rule ${regCtx.ruleName} :  option 'absunits' threshold '${th}' is not a number`);b=true}}}this._unitOpts=units}if(sels=customOpts["selectors"]){if(!regCtx.utils.utils.isObject(sels)){regCtx.utils.msgLib.error(`Rule ${regCtx.ruleName} :  option 'selectors' is not an object`);this._fail=true;return false}accept=sels.accept;ign=sels.ignore;if(accept&&ign){regCtx.utils.msgLib.error(`Rule ${regCtx.ruleName} :  option 'selectors' - sub-properties 'accept' and 'ignore are mutually exclusive`);b=true}if(!b){if(accept){this._accept=this._getOptionSels(regCtx,"accept",accept)}else if(ign){this._ignore=this._getOptionSels(regCtx,"ignore",ign)}}}return this._fail?false:!b}_getOptionSels(regCtx,prop,val){var i;if(typeof val==="string"){val=[val]}else if(!Array.isArray(val)){return null}for(i=0;i<val.length;i++){if(typeof val[i]==="string"){try{val[i]=new RegExp(val[i])}catch(e){regCtx.utils.msgLib.error(`Rule ${regCtx.ruleName} '${prop}' :  invalid reg. expression '${val[i]}' at position ${i}`);this._fail=true}}}return this._fail?null:val}}function _getSelectors(rule){var sels,sel,ch,selch,prel,i,j;prel=rule.prelude;if(prel&&prel.type===SELECTORLIST&&prel.children){ch=prel.children;for(i=0;i<ch.length;i++){sel=ch[i];if(sel.type===SELECTOR&&sel.children){selch=sel.children;for(j=0;j<selch.length;j++){sel=selch[j];if(sel.type===CLASS_SELECTOR||sel.type===ID_SELECTOR){if(!sels){sels=[]}sels.push(sel.name)}}}}}return sels};function _isCssVar(decl){var c,v;c=decl.value.children;if(c){v=c[0];return v.name==="var"}return false};function _isColorFunc(fnName){fnName=fnName.toLowerCase();return fnName.startsWith("rgb")||fnName.startsWith("hsl")};module.exports=Rule;