/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const RULE_NAME="oj-css-style-override";const SHORT_DESCRIPT="JET CSS style classes should not be overridden";const DESCRIPT="JET CSS style classes should not be overridden.<br>JET tag selectors are not "+"considered to be overrides if the CSS rule body sets CSS variables only "+"(i.e. does not set any real properties).";const SELECTOR_LIST="SelectorList";const CLASS_SELECTOR="ClassSelector";const TYPE_SELECTOR="TypeSelector";const PSEUDO_CLASS_SELECTOR="PseudoClassSelector";const NUMBER="number";const STRING="string";const NOT_CLASSES=["oj-complete"];const OVERRIDE_LIST=["oj-buttonset-one","oj-buttonset-many","oj-color-invert","oj-badge-sm","oj-navigationlist-vertical","oj-navigationlist-horizontal","oj-navigationlist-icon-only","oj-navigationlist-stack-icon-label","oj-input-search-hero","oj-avatar-bg-purple","oj-avatar-bg-orange","oj-avatar-bg-teal","oj-avatar-bg-green","oj-avatar-bg-pink","oj-avatar-bg-slate","oj-avatar-bg-blue","oj-avatar-bg-lilac","oj-avatar-bg-gray","oj-avatar-xxs","oj-avatar-xs","oj-avatar-sm","oj-avatar-lg","oj-avatar-xl","oj-avatar-xxl","oj-tabbar-vertical","oj-tabbar-horizontal","oj-tabbar-icon-only","oj-tabbar-stack-icon-label","oj-invalid-drop"];const OVERRIDE_LIST_PREFIX=["oj-animate-inline-"];const NO_OVERRIDE=["oj-removable"];function Rule(){this._checkList=null;this._isAccept=false};Rule.prototype.getName=function(){return RULE_NAME};Rule.prototype.getShortDescription=function(){return SHORT_DESCRIPT};Rule.prototype.getDescription=function(){return DESCRIPT};Rule.prototype.register=function(regCtx){if(!this._checkAcceptRejectOption(regCtx)){return false}this._metaLib=regCtx.utils.metaLib;if(!this._metaLib.hasJetStylePrefixes()){return false}return{"css-rule":this._onRule}};Rule.prototype._onRule=function(ruleCtx,rule){var sels,sel,emit,t,i;sels=this._getOjSelectors(rule,this._metaLib);if(!sels.length){return}for(i=0;i<sels.length;i+=3){sel=sels[i];t=sels[i+1];if(_isBuiltinOverride(sel)){continue}emit=false;if(this._checkList){if(this._isSelAccepted(sel)){emit=true}}else if(t===CLASS_SELECTOR){emit=true}else if(t===TYPE_SELECTOR&&this._metaLib.isJetTag(sel)){if(!this._setsCssVarsOnly(rule.block)){emit=true}}if(emit){this._emitIssue(ruleCtx,t,sel,sels[i+2])}};};Rule.prototype._emitIssue=function(ruleCtx,type,sel,loc){var tag,issue;if(type===TYPE_SELECTOR){issue=new ruleCtx.Issue(`type Selector '${sel}' must not be overridden.`);issue.setMsgKey(ruleCtx.ruleName+"_2")}else{tag=ruleCtx.utils.metaLib.getJetTagFromStyle(sel);issue=new ruleCtx.Issue(`class selector '.${sel}' ${tag?"(for <"+tag+">) ":""}must not be overridden.`);issue.setMsgKey(ruleCtx.ruleName+"_1")}issue.setPosition(ruleCtx.utils.CssUtils.getPosition(ruleCtx,loc));ruleCtx.reporter.addIssue(issue,ruleCtx)};Rule.prototype._setsCssVarsOnly=function(block){var childs=block.children,propset=false;childs.forEach(stmt=>{if(stmt&&stmt.hasOwnProperty("property")){if(!stmt.property.startsWith("--")){propset=true}}});return!propset};Rule.prototype._getOjSelectors=function(rule){var sels,sel,a,c,i,j,ret=[];if(!(rule.prelude&&rule.prelude.children)||rule.prelude.type!=="SelectorList"){return ret}a=rule.prelude.children;for(i=0;i<a.length;i++){sels=a[i];if(sels.type==="Selector"){c=sels.children;for(j=0;j<c.length;j++){sel=c[j];if(sel.type===CLASS_SELECTOR&&(this._metaLib.isJetStylePrefix(sel.name)||NO_OVERRIDE.includes(sel.name))||sel.type===TYPE_SELECTOR&&this._metaLib.isJetTag(sel.name)){ret.push(sel.name);ret.push(sel.type);ret.push(sel.loc)}else if(sel.type===PSEUDO_CLASS_SELECTOR){this._evalPseudoClassSelector(sel,ret,sel.name)}}}}return ret};Rule.prototype._evalPseudoClassSelector=function(sel,a,pseudo){var c,o,l,e,i,j;if(!(c=sel.children)){return}for(i=0;i<c.length;i++){o=c[i];if(o.type==="Selector"){if(l=o.children){for(j=0;j<l.length;j++){e=l[j];if(e.type===CLASS_SELECTOR&&this._metaLib.isJetStylePrefix(e.name)||e.type===TYPE_SELECTOR&&this._metaLib.isJetTag(e.name)){if(!_checkNotClasses(e.name)){a.push(e.name);a.push(e.type);a.push(e.loc)}}}}continue}else{if(o.type!==SELECTOR_LIST){continue}}this._evalPseudoClassSelector(o,a,pseudo)}};Rule.prototype._isSelAccepted=function(sel){if(this._checkList){for(let i=0;i<this._checkList.length;i++){if(this._checkList[i].test(sel)){return this._isAccept}}return!this._isAccept}return true};Rule.prototype._checkAcceptRejectOption=function(regCtx){var customOpts,accept,reject;if(!(customOpts=regCtx.ruleOpts.customOpts)){return true}accept=customOpts.accept;reject=customOpts.reject;if(accept||reject){if(accept&&reject){regCtx.utils.msgLib.error(`Rule ${RULE_NAME}:  override options 'accept' and 'reject' are mutually exclusive`);return false}if(accept){this._checkList=accept;this._isAccept=true}else{this._checkList=reject}if(!Array.isArray(this._checkList)){regCtx.utils.msgLib.error(`Rule ${RULE_NAME}:  option '${accept?"accept":"reject"}' is not an array`);return false}for(let i=0;i<this._checkList.length;i++){if(typeof this._checkList[i]===STRING){if(!(this._checkList[i]=this._makeRegExp(regCtx,this._checkList[i].trim(),i))){return false}}}}return true};Rule.prototype._makeRegExp=function(ruleCtx,s,x){var re;try{re=new RegExp(s.trim())}catch(e){re=null}if(!re){var s=typeof x===NUMBER?`element ${x} `:"";ruleCtx.utils.msgLib.error(`Rule ${RULE_NAME}: in override option '${this._isAccept?"accept":"reject"}' element ${x} is not a valid regular expression`)}return re};function _isBuiltinOverride(sel){if(OVERRIDE_LIST.includes(sel)){return true}for(let i=0;i<OVERRIDE_LIST_PREFIX.length;i++){if(sel.startsWith(OVERRIDE_LIST_PREFIX[i])){return true}}return false};function _checkNotClasses(name){return NOT_CLASSES.includes(name)};module.exports=Rule;