/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const getParentNodeForSlot=require("./helpers/has-slot-name");const RULE_NAME="oj-html-slot-pref-content";const DESCRIPT="When component metadata defines 'preferredContent' for a slot, child elements placed into that slot should "+"implement one or more of the requested interfaces and document that through their 'implements' metadata. "+"This rule applies to components that are directly allocated to a slot, or which are direct children "+"of a template allocated to that slot.";const SHORT=DESCRIPT;const IGNORE_TAG=["oj-defer","html","head","body","script"];const FLATTENABLE_TAGS=["div","oj-bind-if","oj-bind-for-each","oj-defer","template","oj-refresher","oj-validation-group","span","p"];const TEMPLATE="template";const OJ_SWITCHER="oj-switcher";const OJ_IF="oj-if";const OJ_DEFER="oj-defer";const OJ_BIND="oj-bind";const OJ_TOOLBAR="oj-toolbar";const OJ_BIND_SLOT="oj-bind-slot";const OJ_MODULE="oj-module";const OJ_VB_FRAGMENT="oj-vb-fragment";const DIV="div";const SPAN="span";const TAG="tag";const SLOT="slot";const _isBindTag=tagName=>{return tagName.startsWith("oj-bind")};const _ignoreTag=tagName=>{return IGNORE_TAG.includes(tagName)||tagName.startsWith("oj-bind")};const _isExempt=tagName=>{return tagName===OJ_VB_FRAGMENT||tagName===OJ_MODULE};function Rule(){};Rule.prototype.getName=function(){return RULE_NAME};Rule.prototype.getShortDescription=function(){return SHORT};Rule.prototype.getDescription=function(){return DESCRIPT};Rule.prototype.register=function(regCtx){this._metaLib=regCtx.utils.metaLib;this._hasJetWCInterfaces=this._metaLib.hasJetWCInterfaces();this._utils=regCtx.utils.utils;this._domUtils=regCtx.utils.DomUtils;return{"webcomp":this._onTag,"elem":this._onElem}};Rule.prototype._onElem=function(ruleCtx,elemName){if(ruleCtx.utils.DomUtils.isCommonElem(elemName)||elemName===TEMPLATE){if(!_ignoreTag(elemName)){return this._onTag(ruleCtx,elemName)}}};Rule.prototype._onTag=function(ruleCtx,tagName){var attribs=ruleCtx.tagNode.attribs;var slotx,slotName="";this._slotDefined=false;if(slotx=this._hasSlot(ruleCtx.tagNode)){slotName=slotx;this._slotDefined=true}this._onSlot(ruleCtx,slotName)};Rule.prototype._onSlot=function(ruleCtx,slotName){var pnode,slotTag,psc;if(slotName&&ruleCtx.tagNode.name===OJ_BIND_SLOT){return}if(!(pnode=getParentNodeForSlot(ruleCtx,slotName))){return}if(!this._metaLib.isWCTag(pnode.name)){return}if(this._metaLib.isJetTag(pnode.name)&&!this._hasJetWCInterfaces){return}psc=this._metaLib.getPreferredSlotContentEx(pnode.name,slotName);if(!(psc&&psc.interfaces)){return}slotTag=ruleCtx.tag;if(_isExempt(slotTag)){return}if(slotTag===DIV||slotTag===OJ_IF||slotTag===OJ_SWITCHER||slotTag.startsWith(OJ_BIND)||slotTag===OJ_DEFER||slotTag===TEMPLATE){let childs;if(childs=this._flattenDivChildren(psc,pnode,ruleCtx.tagNode,slotName)){let child;for(let i=0;i<childs.length;i++){child=childs[i];if(!_isExempt(child.name)){if(this._hasSlot(child)){continue}if(!this._isSlotInterfaceSupported(child,psc)){this._emitIssue(ruleCtx,child,slotName,pnode.name,psc,4)}}};return}else{this._emitIssue(ruleCtx,ruleCtx.tagNode,slotName,pnode.name,psc,2)}if(slotTag===OJ_SWITCHER||slotTag===OJ_IF){return}}if(ruleCtx.utils.DomUtils.isCommonElem(slotTag)){if(pnode.name!==OJ_TOOLBAR&&ruleCtx.tagNode.name!==SPAN){this._emitIssue(ruleCtx,ruleCtx.tagNode,slotName,pnode.name,psc,3)}return}if(this._metaLib.isWCTag(slotTag)){if(!_isBindTag(slotTag)&&slotTag!==OJ_DEFER&&slotTag!==TEMPLATE){if(!this._isSlotInterfaceSupported(ruleCtx.tagNode,psc)){this._emitIssue(ruleCtx,ruleCtx.tagNode,slotName,pnode.name,psc,1)}}}};Rule.prototype._flattenDivChildren=function(psc,pnode,tagNode,slotName,children){var nodes=tagNode.children;var node,tagName,i;for(i=0;i<nodes.length;i++){node=nodes[i];if(node.type!=="tag"){continue}tagName=node.name;if(FLATTENABLE_TAGS.includes(tagName)){if(!node.attributes?.slot){children=this._flattenDivChildren(psc,pnode,node,slotName,children)}continue}if(!children){children=[]}children.push(node)}return children};Rule.prototype._isSlotInterfaceSupported=function(node,psc){var ret=false,aIfc;if(node){if(aIfc=this._metaLib.getInterfaces(node.name)){for(let ifc of aIfc){if(psc.interfaces.includes(ifc)){ret=true;break}}}}return ret};Rule.prototype._emitIssue=function(ruleCtx,elemNode,slotName,slotOwner,psci,issueType){var ifc=_list(psci.interfaces,"or");var comp=psci.components?` (e.g. ${_list(psci.components,"or",true)})`:"";var multi=psci.interfaces.length>1;var actualSlot=slotName;var slotName=slotName?`slot '${slotName}'`:"default slot";var isDefSlot=!slotName;var issue,msg;if(issueType===1||issueType===4){msg=`<${elemNode.name}> is in${isDefSlot?" the":""} ${slotName} of <${slotOwner}> but does not implement interface${multi?"s":""} ${ifc}${comp}`}else if(issueType===2){msg=`<${elemNode.name}> is in${isDefSlot?" the":""} ${slotName} of <${slotOwner}> but its first child does not implement interface${multi?"s":""} ${ifc}${comp}`}else if(issueType===3){msg=`${slotName} on <${elemNode.name}> for <${slotOwner}> cannot provide ${multi?"interfaces":"an interface"} of type ${ifc}${comp}`}else{return}issue=new ruleCtx.Issue(msg);if(issueType===4){let pos=ruleCtx.utils.DomUtils.getLineCol(ruleCtx.data,elemNode.startIndex);issue.setPosition(pos.row,pos.col,elemNode.startIndex,elemNode.endIndex)}else if(this._slotDefined){let attrPos=ruleCtx.utils.DomUtils.getAttribValuePosition(ruleCtx.data,ruleCtx.tagNode,SLOT);ruleCtx.utils.DomUtils.getAttribPosition(ruleCtx.data,ruleCtx.tagNode,SLOT);issue.setPosition(attrPos.row,attrPos.col,attrPos.start,attrPos.end)}issue.setMsgEx({slotName:actualSlot,slotElem:elemNode.name,slotOwner:slotOwner});ruleCtx.reporter.addIssue(issue,ruleCtx)};Rule.prototype._hasSlot=function(tagNode){var slotName,attribs,slotx;if(attribs=tagNode.attribs){if(this._utils.hasAnyProps(attribs)){slotx=attribs[SLOT];if(slotx||slotx===""){slotName=slotx}}}return slotName};function _getFirstChildTag(childs){var ret,childs,child;if(childs){for(let i=0;i<childs.length;i++){child=childs[i];if(child.type!==TAG){continue}if(child.name.startsWith("oj-bind-")||child.name===OJ_DEFER||child.name===TEMPLATE){return _getFirstChildTag(child.children)}ret=child;break}}return ret};function _list(a,orand,isComp){var s="";for(let i=0;i<a.length;i++){if(i){if(i===a.length-1){s+=(isComp?">":"'")+" "+orand+" "}else{s+=(isComp?">":"'")+", "}}s+=(isComp?"<":"'")+a[i]}return s?s+=isComp?">":"'":s};module.exports=Rule;