/*
 * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 * Licensed under The Universal Permissive License (UPL), Version 1.0
 * as shown at https://oss.oracle.com/licenses/upl/
 */
const helperClass=require("./helpers/vartrack");const RULE_NAME="oj-js-var-undef";var NT;function Rule(){this._reset()};Rule.prototype.getName=function(){return RULE_NAME};Rule.prototype.getShortDescription=function(){return"JS variables should be defined"};Rule.prototype.getDescription=function(){return"Referenced Javascript variables should be defined."};Rule.prototype.register=function(){return{FunctionDeclaration:this._setup,file:this._newFile,startscript:this._newFile,endfile:this._endFile,endscript:this._endFile,CallExpression:this._handleCallExpressions,MemberExpression:this._handleMemberExpressions,AssignmentExpression:this._handleAssignments,FunctionExpression:this._handleFuncExpressions,IfStatement:this._handleIf,NewExpression:this._handleNew,ReturnStatement:this._handleReturns}};Rule.prototype._reset=function(){this._funcVarTable=null;this._globalVars=false;this._define=false;if(this._helper){this._helper.reset()}};Rule.prototype._handleCallExpressions=function(context,node){var n,arg,props,prop,i,j;if(node.callee&&(node.callee.name==="define"||node.callee.name==="require")){this._funcVarTable=this._handleViewModelDefine(context,node);return}n=node.callee;while(n){if(n.type===NT.IDENTIFIER){break}n=n.object}if(n&&n.name){if(!this._funcVarTable[n.name]){this._helper.isVarDefined(context,node,n,n.name)}}if(node.arguments){for(i=0;i<node.arguments.length;i++){arg=node.arguments[i];if(arg&&arg.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,arg,arg.name)}else if(arg.type===NT.OBJ_EXPR){props=arg.properties;for(j=0;j<props.length;j++){prop=props[j];if(prop.type===NT.PROPERTY){if(prop.value.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,prop.value,prop.value.name)}}}}}}};Rule.prototype._handleMemberExpressions=function(context,node){var n,v;if(node.parent&&node.parent.type===NT.NEW_EXPR){return}if(node.object&&node.object.type===NT.IDENTIFIER&&node.property.name!=="prototype"){if(node.object.name){this._helper.isVarDefined(context,node,node.object,node.object.name);return}}if(node.object){n=node.object;while(n){if(n.property&&n.property.type===NT.IDENTIFIER&&n.property.name==="prototype"){return}if(n.type===NT.IDENTIFIER){v=n.name;break}n=n.object}}else if(node.callee){n=node.callee;while(n){if(n.type===NT.IDENTIFIER){v=n.name;break}n=n.object}}if(v){if(!this._funcVarTable[v]){this._helper.isVarDefined(context,node,n,v)}}};Rule.prototype._handleAssignments=function(context,node){var n;if(node.left.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.left,node.left.name)}if(node.right.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.right,node.right.name)}if(node.right.type===NT.CONDITIONAL_EXPR){if(node.right.test.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.right,node.right.test.name)}if(node.right.alternate.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.right.alternate,node.right.alternate.name)}if(node.right.consequent.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.right.consequent,node.right.consequent.name)}}if(node.left.type===NT.MEMBER_EXPR){n=node.left.object;while(n){if(n.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,n,n.name);break}n=n.object}}if(node.right.type===NT.MEMBER_EXPR){n=node.right.object;while(n){if(n.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,n,n.name);break}n=n.object}}if(node.left.object&&node.left.object.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.left.object,node.left.object.name)}if(node.right.object&&node.right.object.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.right.object,node.right.object.name)}};Rule.prototype._handleIf=function(context,node){var n,body,stmt;if(node.test){if(node.test.type===NT.BINARY_EXPR){if(node.test.left.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.test.left,node.test.left.name)}if(node.test.right.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.test.right,node.test.right.name)}return}else if(node.test.type===NT.MEMBER_EXPR){n=node.test.object;while(n){if(n.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,n,n.name);return}n=n.object}}else if(node.test.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.test,node.test.name)}}if(node.consequent){n=node.consequent;if(n.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,n,n.name);return}n=n.object;while(n){if(n.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,n,n.name);return}n=n.object}if(node.consequent.body){body=node.consequent.body;for(stmt in body){stmt=body[stmt];this._helper.pushNodeWithParent(node);this._dispatchHandler(context,stmt);this._helper.popNodeWithParent()}}}if(node.alternate){n=node.alternate;if(n.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,n,n.name);return}n=n.object;while(n){if(n.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,n,n.name);return}n=n.object}if(node.alternate.body){body=node.alternate.body;for(stmt in body){stmt=body[stmt];this._helper.pushNodeWithParent(node);this._dispatchHandler(context,stmt);this._helper.popNodeWithParent()}}}};Rule.prototype._handleFuncExpressions=function(context,node){if(!this._helper.isFunctionKnown(context,node)){this._helper.createFuncDeclarations(context,node)}};Rule.prototype._handleExpressions=function(context,node){var expr=node.expression;if(expr.type===NT.ASSIGNMENT_EXPR){this._handleAssignments(context,expr)}};Rule.prototype._handleReturns=function(context,node){var n,args,arg,i;if(node.argument){if(node.argument.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,node.argument,node.argument.name)}else if(node.argument.type===NT.MEMBER_EXPR){n=n.argument.object;while(n){if(n.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,n,n.name);return}n=n.object}}else if(node.argument.type===NT.NEW_EXPR){this._handleNew(context,node)}else if(node.argument.type===NT.CALL_EXPR){args=node.argument.arguments;for(i=0;i<args.length;i++){arg=args[i];if(arg.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,arg,arg.name)}}}}};Rule.prototype._handleNew=function(context,node){var i,arg;if(node.arguments){for(i=0;i<node.arguments.length;i++){arg=node.arguments[i];if(arg.type===NT.IDENTIFIER){this._helper.isVarDefined(context,node,arg,arg.name)}}}};Rule.prototype._dispatchHandler=function(context,node){if(node.type===NT.MEMBER_EXPR){this._handleMemberExpressions(context,node)}else if(node.type===NT.ASSIGNMENT_EXPR){this._handleAssignments(context,node)}else if(node.type===NT.CALL_EXPR){this._handleCallExpressions(context,node)}else if(node.type===NT.EXPR_STMT){this._handleExpressions(context,node)}else{var jj=42}};Rule.prototype._newFile=function(context){if(context.type==="file"&&context.filetype==="html"){return}this._globalVars=this._funcVarTable=null;this._define=false;if(!this._helper){this._helper=new helperClass}this._reset();if(!NT&&context.utils.AstUtils){NT=context.utils.AstUtils.getNodeTypes()}};Rule.prototype._endFile=function(context){var funcName;var undeclared=this._helper.getUndeclared();if(this._funcVarTable){for(funcName in this._funcVarTable){this._reportUnusedVars(context,this._funcVarTable[funcName])}this._reportUndeclared(context,this._funcVarTable[funcName],undeclared)}this._reset()};Rule.prototype._reportUnusedVars=function(context,funcVars){var varName,varObj,issue,rowcol;for(varName in funcVars){varObj=funcVars[varName];if((varObj.type==="var"||varObj.type==="globvar")&&!funcVars[varName].used){issue=new context.Issue("Unreferenced var '"+varName+"'");rowcol=context.utils.utils.getRowColFromIndex(context.rawData,varObj.start);issue.setPosition(rowcol.row,rowcol.col,varObj.start,varObj.start+varName.length-1);issue.setSeverity("info");issue.setMsgKey(context.ruleName+"_1");context.reporter.addIssue(issue,context)}}};Rule.prototype._reportUndeclared=function(context,funcVars,undeclared){var varName,funcName,node,issue,rowcol,entry,x;for(entry in undeclared){node=undeclared[entry];x=entry.indexOf("!@#$%");varName=entry.substring(0,x);funcName=this._helper.getContainingFunctionName(context,node);if(funcName){funcVars=this._funcVarTable[funcName];if(funcVars){if(funcVars[varName]){if(funcVars[varName.used]){continue}}}}issue=new context.Issue("Undeclared var '"+varName+"'");rowcol=context.utils.utils.getRowColFromIndex(context.rawData,node.start);issue.setPosition(rowcol.row,rowcol.col,node.start,node.start+varName.length-1);issue.setSeverity("info");issue.setMsgKey(context.ruleName+"_1");context.reporter.addIssue(issue,context)}};Rule.prototype._setup=function(context,node){if(!NT){NT=context.utils.AstUtils.getNodeTypes()}if(!this._globalVars){this._helper.getGlobalVars(context);this._globalVars=true}if(this._define){return}var o=this._helper.createFuncDeclarations(context,node);this._funcVarTable=o.funcVarTable};Rule.prototype._handleViewModelDefine=function(context,node){var args,stmt,o,i;args=node.arguments;if(!args){return}for(i=0;i<args.length;i++){stmt=args[i];if(stmt.type===NT.FUNC_EXPR){o=this._helper.createFuncDeclarations(context,stmt);this._funcVarTable=o.funcVarTable}}this._define=true;return this._funcVarTable};module.exports=Rule;